@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2001 Stuart Parmenter and others,
+ * Copyright (C) 1997-2009 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,11 +20,13 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "abook-completion.h"
 
 #include <string.h>
 
-#include "abook-completion.h"
 #include "information.h"
 #include "misc.h"
 
@@ -46,7 +48,7 @@ completion_data_new(InternetAddress * ia, const gchar * nick_name)
 
     ret = g_new0(CompletionData, 1);
 
-    internet_address_ref(ia);
+    g_object_ref(ia);
     ret->ia = ia;
 
     string = g_string_new(nick_name);
@@ -78,7 +80,7 @@ completion_data_new(InternetAddress * ia, const gchar * nick_name)
 void
 completion_data_free(CompletionData * data)
 {
-    internet_address_unref(data->ia);
+    g_object_unref(data->ia);
     g_free(data->string);
     g_free(data);
 }
