@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2003 Stuart Parmenter and others,
+ * Copyright (C) 1997-2009 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -32,12 +32,15 @@
  * organizationalPerson type. We are lenient.
  *
  */
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 
 #if ENABLE_LDAP
 
+#include "address-book-ldap.h"
+
 #include <glib.h>
-#include <gnome.h>
 #include <sys/time.h>
 #include <string.h>
 #include <lber.h>
@@ -49,7 +52,6 @@
 #endif
 
 #include "address-book.h"
-#include "address-book-ldap.h"
 #include "information.h"
 #include "libbalsa-conf.h"
 #include <glib/gi18n.h>
@@ -547,7 +549,7 @@ lbabl_get_internet_address(LDAP *dir, LDAPMessage * e)
 
     if(!cn)
         cn = create_name(first, sn);
-    ia = internet_address_new_name(cn, email);
+    ia = internet_address_mailbox_new(cn, email);
     g_free(email); g_free(sn); g_free(cn); g_free(first);
 
     return ia;
@@ -604,6 +606,12 @@ libbalsa_address_book_ldap_add_address(LibBalsaAddressBook *ab,
 	    return LBABERR_CANNOT_CONNECT;
     }
 
+    if(ldap_ab->priv_book_dn == NULL) {
+        libbalsa_address_book_set_status
+            (ab, _("Undefined location of user address book"));
+        return LBABERR_CANNOT_WRITE;
+    }                                    
+
     dn = g_strdup_printf("mail=%s,%s",
                          (char*)address->address_list->data,
                          ldap_ab->priv_book_dn);
@@ -904,7 +912,7 @@ rfc_2254_escape(const gchar *raw)
     gchar *str;
     gchar *step;
 
-    new = (gchar *)malloc((strlen(raw) * 3) + 1);
+    new = g_new(gchar, (strlen(raw) * 3) + 1);
     str = new;
     for (step = (gchar *)raw;
          step[0] != '\0';
@@ -936,7 +944,7 @@ rfc_2254_escape(const gchar *raw)
     }
     str[0] = '\0';
     str = g_strdup(new);
-    free(new);
+    g_free(new);
     return str;
 }
 
