@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2007 Stuart Parmenter and others,
+ * Copyright (C) 1997-2009 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,11 @@
  */
 
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "address-view.h"
+
 #include <string.h>
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
@@ -35,7 +39,6 @@
 /*
  * LibBalsa includes.
  */
-#include "address-view.h"
 #include "address-book.h"
 #include "cell-renderer-button.h"
 #include "misc.h"
@@ -227,7 +230,7 @@ lbav_entry_setup_matches(LibBalsaAddressView * address_view,
     if (*prefix)
         match = lbav_get_matching_addresses(address_view, prefix, type);
     lbav_append_addresses(address_view, completion, match, prefix);
-    g_list_foreach(match, (GFunc) internet_address_unref, NULL);
+    g_list_foreach(match, (GFunc) g_object_unref, NULL);
     g_list_free(match);
 }
 
@@ -370,30 +373,26 @@ lbav_add_from_list(LibBalsaAddressView * address_view,
         gtk_tree_view_get_model(GTK_TREE_VIEW(address_view));
     GtkListStore *address_store = GTK_LIST_STORE(model);
     guint type;
+    int i;
 
     gtk_tree_model_get(model, iter, ADDRESS_TYPE_COL, &type, -1);
 
-    while (list) {
-        InternetAddress *ia = list->address;
+    for (i = 0; i < internet_address_list_length(list); i++) {
+        InternetAddress *ia = internet_address_list_get_address(list, i);
+        gchar *name = internet_address_to_string(ia, FALSE);
 
-        if (ia) {
-            gchar *name = internet_address_to_string(ia, FALSE);
+        libbalsa_utf8_sanitize(&name, address_view->fallback, NULL);
+        lbav_clean_text(name);
 
-            libbalsa_utf8_sanitize(&name, address_view->fallback, NULL);
-            lbav_clean_text(name);
-
-            gtk_list_store_set(address_store, iter,
-                               ADDRESS_TYPE_COL, type,
-                               ADDRESS_TYPESTRING_COL,
-                               _(lbav_type_string(address_view, type)),
-                               ADDRESS_NAME_COL, name,
-                               ADDRESS_ICON_COL, lbav_close_icon,
-                               -1);
-            g_free(name);
-        }
-
-        if ((list = list->next))
+        if (i > 0)
             gtk_list_store_insert_after(address_store, iter, iter);
+        gtk_list_store_set(address_store, iter,
+                           ADDRESS_TYPE_COL, type,
+                           ADDRESS_TYPESTRING_COL,
+                           _(lbav_type_string(address_view, type)),
+                           ADDRESS_NAME_COL, name,
+                           ADDRESS_ICON_COL, lbav_close_icon, -1);
+        g_free(name);
     }
 }
 
@@ -407,12 +406,16 @@ static gboolean
 lbav_add_from_string(LibBalsaAddressView * address_view,
                      GtkTreeIter * iter, const gchar * string)
 {
-    InternetAddressList *list = internet_address_parse_string(string);
+    InternetAddressList *list = internet_address_list_parse_string(string);
+    gboolean retval = FALSE;
 
-    lbav_add_from_list(address_view, iter, list);
-    internet_address_list_destroy(list);
+    if (list) {
+        if ((retval = (internet_address_list_length(list) > 0)))
+            lbav_add_from_list(address_view, iter, list);
+        g_object_unref(list);
+    }
 
-    return list != NULL;
+    return retval;
 }
 
 /*
@@ -533,24 +536,6 @@ lbav_set_text_at_path(LibBalsaAddressView * address_view,
     }
 }
 
-/*
- *     Count addresses in an InternetAddressList
- */
-static void
-lbav_count_addresses_in_list(InternetAddressList * list, gint * addresses)
-{
-    for (; list && *addresses >= 0; list = list->next) {
-        InternetAddress *ia = list->address;
-        if (ia->type == INTERNET_ADDRESS_NAME) {
-            if (strpbrk(ia->value.addr, "@%!"))
-                ++(*addresses);
-            else
-                *addresses = -1;
-        } else if (ia->type == INTERNET_ADDRESS_GROUP)
-            lbav_count_addresses_in_list(ia->value.members, addresses);
-    }
-}
-
 /*
  *     Find an address_type
  */
@@ -765,7 +750,7 @@ lbav_focus_out_cb(GtkEntry * entry, GdkEventFocus * event,
                 gtk_cell_editable_editing_done(GTK_CELL_EDITABLE(entry));
                 g_free(the_addr);
             }
-            g_list_foreach(match, (GFunc) internet_address_unref, NULL);
+            g_list_foreach(match, (GFunc) g_object_unref, NULL);
             g_list_free(match);
         }
     }
@@ -945,11 +930,7 @@ lbav_selection_changed_cb(GtkTreeSelection * selection,
                 GtkTreePath *path;
                 GtkTreeViewColumn *column;
 
-#if GTK_CHECK_VERSION(2, 12, 0)
                 gtk_tree_view_convert_widget_to_bin_window_coords
-#else                           /* GTK_CHECK_VERSION(2, 12, 0) */
-                gtk_tree_view_widget_to_tree_coords
-#endif                          /* GTK_CHECK_VERSION(2, 12, 0) */
                     (tree_view, (gint) event_button->x,
                      (gint) event_button->y, &x, &y);
 
@@ -1190,7 +1171,7 @@ libbalsa_address_view_set_from_list(LibBalsaAddressView * address_view,
 
     lbav_remove(address_view, type);
 
-    if (list) {
+    if (list && internet_address_list_length(list) > 0) {
         GtkTreeModel *model =
             gtk_tree_view_get_model(GTK_TREE_VIEW(address_view));
         GtkListStore *address_store = GTK_LIST_STORE(model);
@@ -1213,19 +1194,18 @@ libbalsa_address_view_set_from_list(LibBalsaAddressView * address_view,
 gint
 libbalsa_address_view_n_addresses(LibBalsaAddressView * address_view)
 {
-    gint addresses;
+    gint addresses = 0;
     guint type;
 
     g_return_val_if_fail(LIBBALSA_IS_ADDRESS_VIEW(address_view), -1);
 
-    addresses = 0;
     for (type = 0; type < address_view->n_types; type++) {
         InternetAddressList *list =
             libbalsa_address_view_get_list(address_view,
                                            lbav_type_string(address_view,
                                                             type));
-        lbav_count_addresses_in_list(list, &addresses);
-        internet_address_list_destroy(list);
+        addresses += libbalsa_address_n_mailboxes_in_list(list);
+        g_object_unref(list);
     }
 
     return addresses;
@@ -1233,7 +1213,8 @@ libbalsa_address_view_n_addresses(LibBalsaAddressView * address_view)
 
 /*
  *     Create InternetAddressList corresponding to the view content.
- *     The list must be destroyed using internet_address_list_destroy().
+ *     The list, which is NULL only on error, must be destroyed using
+ *     g_object_unref().
  */
 InternetAddressList *
 libbalsa_address_view_get_list(LibBalsaAddressView * address_view,
@@ -1252,7 +1233,7 @@ libbalsa_address_view_get_list(LibBalsaAddressView * address_view,
                          || type < address_view->n_types, NULL);
 
     model = gtk_tree_view_get_model(GTK_TREE_VIEW(address_view));
-    address_list = NULL;
+    address_list = internet_address_list_new();
     for (valid = gtk_tree_model_get_iter_first(model, &iter);
          valid; valid = gtk_tree_model_iter_next(model, &iter)) {
         guint this_type;
@@ -1263,15 +1244,10 @@ libbalsa_address_view_get_list(LibBalsaAddressView * address_view,
                            ADDRESS_NAME_COL, &name, -1);
 
         if (this_type == type && name && *name) {
-            InternetAddressList *l, *tmp_list =
-                internet_address_parse_string(name);
-            for (l = tmp_list; l; l = l->next) {
-                InternetAddress *ia = l->address;
-                if (ia)
-                    address_list =
-                        internet_address_list_append(address_list, ia);
-            }
-            internet_address_list_destroy(tmp_list);
+            InternetAddressList *tmp_list =
+                internet_address_list_parse_string(name);
+            internet_address_list_append(address_list, tmp_list);
+            g_object_unref(tmp_list);
         }
         g_free(name);
     }
