@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2002 Stuart Parmenter and others,
+ * Copyright (C) 1997-2009 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,14 +24,17 @@
  * filter-funcs.c
  */
 
-#include "config.h"
-#define _XOPEN_SOURCE 500
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "filter-funcs.h"
+
 #include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include "missing.h"
 
-#include "filter-funcs.h"
 #include "filter-private.h"
 
 /* Conditions */
@@ -309,7 +312,7 @@ static void
 cond_to_string(LibBalsaCondition * cond, GString *res)
 {
     char str[80];
-    struct tm date;
+    GDate date;
 
     if(cond->negate)
         g_string_append(res, "NOT ");
@@ -331,14 +334,14 @@ cond_to_string(LibBalsaCondition * cond, GString *res)
     case CONDITION_DATE:
         g_string_append(res, "DATE ");
 	if (cond->match.date.date_low) {
-	    localtime_r(&cond->match.date.date_low, &date);
-	    strftime(str,sizeof(str),"%Y-%m-%d", &date);
+	    g_date_set_time_t(&date, cond->match.date.date_low);
+	    g_date_strftime(str, sizeof(str), "%Y-%m-%d", &date);
 	} else str[0]='\0';
         append_quoted_string(res, str);
         g_string_append_c(res, ' ');
 	if (cond->match.date.date_high) {
-	    localtime_r(&cond->match.date.date_high, &date);
-	    strftime(str,sizeof(str),"%Y-%m-%d", &date);
+	    g_date_set_time_t(&date, cond->match.date.date_high);
+	    g_date_strftime(str, sizeof(str), "%Y-%m-%d", &date);
 	} else str[0]='\0';
         append_quoted_string(res, str);
 	break;
