@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2002 Stuart Parmenter and others,
+ * Copyright (C) 1997-2009 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -29,19 +29,23 @@
  * Primary author: Emmanuel ALLAUD
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "filter.h"
 
 #include <ctype.h>
 #include <string.h>
 
-#ifdef HAVE_GNOME
+#if HAVE_CANBERRA
+#include <canberra-gtk.h>
+#elif HAVE_GNOME
 #include <libgnome/gnome-sound.h>
-#endif
+#endif                          /* HAVE_CANBERRA */
 
 #include "libbalsa.h"
 #include "libbalsa_private.h"
 
-#include "filter.h"
 #include "filter-funcs.h"
 #include "filter-private.h"
 #include "misc.h"
@@ -247,14 +251,22 @@ libbalsa_filter_mailbox_messages(LibBalsaFilter * filt,
     gboolean result=FALSE;
     LibBalsaMailbox *mbox;
     GError *err = NULL;
+    gchar **parts, **p;
 
     if (msgnos->len == 0)
 	return FALSE;
 
-#ifdef HAVE_GNOME
+#if HAVE_CANBERRA
+    if (filt->sound) {
+        gint rc = ca_context_play(ca_gtk_context_get(), 0,
+                                  CA_PROP_MEDIA_FILENAME, filt->sound,
+                                  NULL);
+        g_message("(%s) play %s, %s", __func__, filt->sound, ca_strerror(rc));
+    }
+#elif HAVE_GNOME
     if (filt->sound)
 	gnome_sound_play(filt->sound);
-#endif /* HAVE_GNOME */
+#endif                          /* HAVE_CANBERRA */
     if (filt->popup_text)
 	libbalsa_information(LIBBALSA_INFORMATION_MESSAGE,
 			     "%s",
@@ -299,6 +311,18 @@ libbalsa_filter_mailbox_messages(LibBalsaFilter * filt,
 	else if (mbox == filters_trash_mbox)
 	    result = TRUE;
 	break;
+    case FILTER_COLOR:
+        parts = g_strsplit(filt->action_string, ";", 2);
+        for (p = parts; *p; p++) {
+            if (g_str_has_prefix(*p, "foreground:"))
+                libbalsa_mailbox_set_foreground(mailbox, msgnos,
+                                                (*p) + 11);
+            if (g_str_has_prefix(*p, "background:"))
+                libbalsa_mailbox_set_background(mailbox, msgnos,
+                                                (*p) + 11);
+        }
+        g_strfreev(parts);
+        break;
     case FILTER_PRINT:
 	/* FIXME : to be implemented */
 	break;
@@ -306,6 +330,7 @@ libbalsa_filter_mailbox_messages(LibBalsaFilter * filt,
 	/* FIXME : to be implemented */
 	break;
     case FILTER_NOTHING:
+    case FILTER_N_TYPES:
 	/* Nothing to do */
 	break;
     }
