@@ -35,13 +35,14 @@
  * more complete API.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "html.h"
 
 #include <stdio.h>
 #include <string.h>
 
-#include "html.h"
-
 #ifdef HAVE_GTKHTML
 
 /* We need the declaration of LibBalsaMessage, but including "message.h"
@@ -479,79 +480,6 @@ libbalsa_html_copy(GtkWidget * widget)
     gtk_html_copy(GTK_HTML(widget));
 }
 
-/*
- * GtkHTML printing suport.
- */
-
-#if defined(HAVE_GNOME) && !defined(HAVE_GTK_PRINT)
-#if !defined(HAVE_GTKHTML3_GTKPRINT)
-gboolean
-libbalsa_html_can_print(void)
-{
-    return TRUE;
-}
-/*
- * Print the page(s) in the widget with a header and a footer.
- */
-void
-libbalsa_html_print(GtkWidget * widget,
-		    GnomePrintContext * print_context,
-		    gdouble header_height, gdouble footer_height,
-		    LibBalsaHTMLPrintCallback header_print,
-		    LibBalsaHTMLPrintCallback footer_print,
-		    gpointer user_data)
-{
-    gtk_html_print_with_header_footer(GTK_HTML(widget), print_context,
-				      header_height, footer_height,
-				      (GtkHTMLPrintCallback) header_print,
-				      (GtkHTMLPrintCallback) footer_print,
-				      user_data);
-}
-
-/*
- * Return the number of pages that will be printed.
- */
-gint
-libbalsa_html_print_get_pages_num(GtkWidget * widget,
-				  GnomePrintContext * print_context,
-				  gdouble header_height,
-				  gdouble footer_height)
-{
-    return gtk_html_print_get_pages_num(GTK_HTML(widget), print_context,
-					header_height, footer_height);
-}
-#else /* HAVE_GTKHTML3_GTKPRINT */
-gboolean
-libbalsa_html_can_print(void)
-{
-    return FALSE;
-}
-/*
- * Print the page(s) in the widget with a header and a footer.
- */
-void
-libbalsa_html_print(GtkWidget * widget,
-		    GnomePrintContext * print_context,
-		    gdouble header_height, gdouble footer_height,
-		    LibBalsaHTMLPrintCallback header_print,
-		    LibBalsaHTMLPrintCallback footer_print,
-		    gpointer user_data)
-{
-}
-
-/*
- * Return the number of pages that will be printed.
- */
-gint
-libbalsa_html_print_get_pages_num(GtkWidget * widget,
-				  GnomePrintContext * print_context,
-				  gdouble header_height,
-				  gdouble footer_height)
-{
-    return 0;
-}
-#endif /* HAVE_GTKHTML3_GTKPRINT */
-#endif /* defined(HAVE_GNOME) && !defined(HAVE_GTK_PRINT) */
 # else				/* HAVE_GTKHTML3 */
 
 /* Code for GtkHtml-2 */
@@ -687,42 +615,6 @@ libbalsa_html_copy(GtkWidget * widget)
 {
 }
 
-#if defined(HAVE_GNOME) && !defined(HAVE_GTK_PRINT)
-/*
- * HtmlView doesn't support printing.
- */
-gboolean
-libbalsa_html_can_print(void)
-{
-    return FALSE;
-}
-
-/*
- * Do nothing.
- */
-void
-libbalsa_html_print(GtkWidget * widget,
-		    GnomePrintContext * print_context,
-		    gdouble header_height, gdouble footer_height,
-		    LibBalsaHTMLPrintCallback header_print,
-		    LibBalsaHTMLPrintCallback footer_print,
-		    gpointer user_data)
-{
-}
-
-/*
- * Return nothing.
- */
-gint
-libbalsa_html_print_get_pages_num(GtkWidget * widget,
-				  GnomePrintContext * print_context,
-				  gdouble header_height,
-				  gdouble footer_height)
-{
-    return 0;
-}
-#endif /* defined(HAVE_GNOME) && !defined(HAVE_GTK_PRINT) */
-
 # endif				/* HAVE_GTKHTML3 */
 
 /* Common code for both widgets. */
@@ -790,7 +682,7 @@ libbalsa_html_filter(LibBalsaHTMLType html_type, gchar ** text, guint len)
     array = g_byte_array_new();
     g_mime_stream_mem_set_byte_array(GMIME_STREAM_MEM(stream), array);
 
-    filter_stream = g_mime_stream_filter_new_with_stream(stream);
+    filter_stream = g_mime_stream_filter_new(stream);
     g_object_unref(stream);
 
     filter = g_mime_filter_enriched_new(flags);
