@@ -19,24 +19,20 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "identity.h"
 
 #ifdef HAVE_GPGME
 #  include "rfc3156.h"
 #endif
 #include "libbalsa.h"
-#include "identity.h"
 #include "information.h"
 #include "libbalsa-conf.h"
 #include <glib/gi18n.h>
 #include "misc.h"
 
-#if !GTK_CHECK_VERSION(2, 14, 0)
-#ifdef HAVE_GNOME
-#include <gnome.h>
-#endif
-#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
-
 #if ENABLE_ESMTP
 #include <string.h>
 #include "smtp-server.h"
@@ -98,7 +94,7 @@ static void
 libbalsa_identity_init(LibBalsaIdentity* ident)
 {
     ident->identity_name = NULL;
-    ident->ia = internet_address_new();
+    ident->ia = NULL;
     ident->replyto = NULL;
     ident->domain = NULL;
     ident->bcc = NULL;
@@ -135,7 +131,8 @@ libbalsa_identity_finalize(GObject * object)
 {
     LibBalsaIdentity *ident = LIBBALSA_IDENTITY(object);
 
-    internet_address_unref(ident->ia);
+    if (ident->ia)
+	g_object_unref(ident->ia);
     g_free(ident->identity_name);
     g_free(ident->replyto);
     g_free(ident->domain);
@@ -203,7 +200,8 @@ libbalsa_identity_set_address(LibBalsaIdentity * ident,
 {
     g_return_if_fail(ident != NULL);
 
-    internet_address_unref(ident->ia);
+    if (ident->ia)
+	g_object_unref(ident->ia);
     ident->ia = ia;
 }
 
@@ -1184,11 +1182,7 @@ file_chooser_check_cb(GtkToggleButton * button, GtkWidget * chooser)
     gtk_widget_set_sensitive(chooser,
                              gtk_toggle_button_get_active(button));
     /* Force validation of current path, if any. */
-#if GTK_CHECK_VERSION(2, 6, 0)
     g_signal_emit_by_name(chooser, "selection-changed");
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    g_signal_emit_by_name(chooser, "changed");
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
 }
 
 static void
@@ -1211,11 +1205,9 @@ md_face_path_changed(const gchar * filename, gboolean active,
                                             filename, &size, &err);
 
     if (err) {
-#if GTK_CHECK_VERSION(2, 6, 0)
         libbalsa_information(LIBBALSA_INFORMATION_WARNING,
                              _("Error reading file %s: %s"), filename,
                              err->message);
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
         g_error_free(err);
         gtk_widget_hide(face_box);
         return;
@@ -1307,11 +1299,7 @@ file_chooser_cb(GtkWidget * chooser, gpointer data)
     GtkToggleButton *check;
     gboolean active;
 
-#if GTK_CHECK_VERSION(2, 6, 0)
     filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    filename = gtk_editable_get_chars(GTK_EDITABLE(chooser), 0, -1);
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
     if (!filename || !*filename) {
         g_free(filename);
         return;
@@ -1338,9 +1326,7 @@ ident_dialog_add_file_chooser_button(GtkWidget * table, gint row,
 {
     GtkWidget *check;
     gchar *filename;
-#if GTK_CHECK_VERSION(2, 6, 0)
     gchar *title;
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
     GtkWidget *button;
 
     check =
@@ -1350,17 +1336,12 @@ ident_dialog_add_file_chooser_button(GtkWidget * table, gint row,
 
     filename =
         g_build_filename(g_get_home_dir(), path_info[type].basename, NULL);
-#if GTK_CHECK_VERSION(2, 6, 0)
     title = g_strdup_printf("Choose %s file", _(path_info[type].info));
     button = gtk_file_chooser_button_new(title,
                                          GTK_FILE_CHOOSER_ACTION_OPEN);
     g_free(title);
     gtk_file_chooser_set_show_hidden(GTK_FILE_CHOOSER(button), TRUE);
     gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(button), filename);
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    button = gtk_entry_new();
-    gtk_entry_set_text(GTK_ENTRY(button), filename);
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
     g_free(filename);
 
     gtk_table_attach(GTK_TABLE(table), button, 1, 2, row, row + 1,
@@ -1372,13 +1353,8 @@ ident_dialog_add_file_chooser_button(GtkWidget * table, gint row,
                       GUINT_TO_POINTER(type));
     g_signal_connect(check, "toggled",
                      G_CALLBACK(file_chooser_check_cb), button);
-#if GTK_CHECK_VERSION(2, 6, 0)
     g_signal_connect(button, "selection-changed",
                      G_CALLBACK(file_chooser_cb), dialog);
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    g_signal_connect(button, "changed",
-                     G_CALLBACK(file_chooser_cb), dialog);
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
 }
 
 static void
@@ -1469,16 +1445,15 @@ ident_dialog_update(GObject * dlg)
     g_free(id->identity_name); id->identity_name = text;
     set_identity_name_in_tree(GTK_TREE_VIEW(tree), id, text);
 
-    text = ident_dialog_get_text(dlg, "identity-fullname");
-    g_return_val_if_fail(text != NULL, FALSE);
-    ia = internet_address_new();
-    internet_address_set_name(ia, text);
-    g_free(text);
-    
     text = ident_dialog_get_text(dlg, "identity-address");
-    internet_address_set_addr(ia, text);
+    g_return_val_if_fail(text != NULL, FALSE);
+    ia = internet_address_mailbox_new(NULL, text);
     g_free(text);
+
+    text = ident_dialog_get_text(dlg, "identity-fullname");
+    internet_address_set_name(ia, text);
     libbalsa_identity_set_address(id, ia);
+    g_free(text);
 
     g_free(id->replyto);
     id->replyto         = ident_dialog_get_text(dlg, "identity-replyto");
@@ -1520,6 +1495,7 @@ ident_dialog_update(GObject * dlg)
     id->warn_send_plain = ident_dialog_get_bool(dlg, "identity-warn-send-plain");
     id->crypt_protocol  = GPOINTER_TO_INT(ident_dialog_get_value
                                           (dlg, "identity-crypt-protocol"));
+    g_free(id->force_key_id);
     id->force_key_id    = g_strstrip(ident_dialog_get_text(dlg, "identity-keyid"));
 #endif
    
@@ -1573,11 +1549,7 @@ ident_dialog_get_path(GObject * dialog, const gchar * key)
     if (!GTK_WIDGET_SENSITIVE(chooser))
         return NULL;
 
-#if GTK_CHECK_VERSION(2, 6, 0)
     return gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    return gtk_editable_get_chars(GTK_EDITABLE(chooser), 0, -1);
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
 }
 
 
@@ -1688,22 +1660,16 @@ delete_ident_response(GtkWidget * confirm, gint response,
 static void
 help_ident_cb(void)
 {
-#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
     GError *err = NULL;
 
-#if GTK_CHECK_VERSION(2, 14, 0)
     gtk_show_uri(NULL, "ghelp:balsa?identities",
                  gtk_get_current_event_time(), &err);
-#else                           /* GTK_CHECK_VERSION(2, 14, 0) */
-    gnome_help_display("balsa", "identities", &err);
-#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
 
     if (err) {
         g_print(_("Error displaying help for identities: %s\n"),
                 err->message);
         g_error_free(err);
     }
-#endif
 }
 
 /* libbalsa_identity_config_dialog displays an identity management
@@ -1850,10 +1816,10 @@ display_frame_update(GObject * dialog, LibBalsaIdentity* ident)
 
     ident_dialog_update(dialog);
     display_frame_set_field(dialog, "identity-name", ident->identity_name);
-    display_frame_set_field(dialog, "identity-fullname", ident->ia->name);
-    if (ident->ia->type == INTERNET_ADDRESS_NAME)
+    display_frame_set_field(dialog, "identity-fullname", ident->ia ? ident->ia->name : NULL);
+    if (ident->ia && INTERNET_ADDRESS_IS_MAILBOX (ident->ia))
         display_frame_set_field(dialog, "identity-address", 
-                                ident->ia->value.addr);
+                                INTERNET_ADDRESS_MAILBOX(ident->ia)->addr);
     else
         display_frame_set_field(dialog, "identity-address", NULL);
     
@@ -1964,17 +1930,16 @@ LibBalsaIdentity*
 libbalsa_identity_new_config(const gchar* name)
 {
     LibBalsaIdentity* ident;
+    gchar *fname, *email;
     gchar* tmpstr;
     
+    fname = libbalsa_conf_get_string("FullName");
+    email = libbalsa_conf_get_string("Address");
+    
     ident = LIBBALSA_IDENTITY(libbalsa_identity_new_with_name(name));
-
-    tmpstr = libbalsa_conf_get_string("FullName");
-    internet_address_set_name(ident->ia, tmpstr);
-    g_free(tmpstr);
-
-    tmpstr = libbalsa_conf_get_string("Address");
-    internet_address_set_addr(ident->ia, tmpstr);
-    g_free(tmpstr);
+    ident->ia = internet_address_mailbox_new (fname, email);
+    g_free(fname);
+    g_free(email);
 
     ident->replyto = libbalsa_conf_get_string("ReplyTo");
     ident->domain = libbalsa_conf_get_string("Domain");
@@ -2025,10 +1990,10 @@ libbalsa_identity_save(LibBalsaIdentity* ident, const gchar* group)
     g_return_if_fail(ident);
 
     libbalsa_conf_push_group(group);
-    libbalsa_conf_set_string("FullName", ident->ia->name);
+    libbalsa_conf_set_string("FullName", ident->ia ? ident->ia->name : NULL);
     
-    if (ident->ia->type == INTERNET_ADDRESS_NAME)
-        libbalsa_conf_set_string("Address", ident->ia->value.addr);
+    if (ident->ia && INTERNET_ADDRESS_IS_MAILBOX (ident->ia))
+        libbalsa_conf_set_string("Address", INTERNET_ADDRESS_MAILBOX(ident->ia)->addr);
 
     libbalsa_conf_set_string("ReplyTo", ident->replyto);
     libbalsa_conf_set_string("Domain", ident->domain);
