@@ -23,14 +23,14 @@
 #ifndef __LIBBALSA_IDENTITY_H__
 #define __LIBBALSA_IDENTITY_H__
 
-#include "config.h"
+#ifndef BALSA_VERSION
+# error "Include config.h before this file."
+#endif
 
 #include <gtk/gtk.h>
 #include <gmime/internet-address.h>
 
-#if ENABLE_ESMTP
-#include "smtp-server.h"
-#endif
+#include "libbalsa.h"
 
 #ifdef __cpluscplus
 extern "C" 
@@ -52,7 +52,6 @@ extern "C"
 #define LIBBALSA_IS_IDENTITY_CLASS(klass) \
     (G_TYPE_CHECK_CLASS_TYPE (klass, LIBBALSA_TYPE_IDENTITY))
 
-    typedef struct _LibBalsaIdentity LibBalsaIdentity;
     typedef struct _LibBalsaIdentityClass LibBalsaIdentityClass;
     
     
@@ -87,6 +86,7 @@ extern "C"
 	gboolean always_trust;
 	gboolean warn_send_plain;
 	gint crypt_protocol;
+        gchar *force_key_id;
 #endif
 #if ENABLE_ESMTP
 	LibBalsaSmtpServer *smtp_server;
