@@ -317,15 +317,18 @@ imap_mbox_subscribe(ImapMboxHandle* handle,
 ImapResponse
 imap_mbox_list(ImapMboxHandle *handle, const char* what)
 {
+  ImapResponse rc;
   IMAP_REQUIRED_STATE2(handle,IMHS_AUTHENTICATED, IMHS_SELECTED, IMR_BAD);
+  HANDLE_LOCK(handle);
   {
   gchar *mbx7 = imap_utf8_to_mailbox(what);
   gchar *cmd = g_strdup_printf("LIST \"%s\" \"%%\"", mbx7);
-  ImapResponse rc = imap_cmd_exec(handle, cmd);
+  rc = imap_cmd_exec(handle, cmd);
   g_free(cmd);
   g_free(mbx7);
-  return rc;
   }
+  HANDLE_UNLOCK(handle);
+  return rc;
 }
 
 
@@ -333,14 +336,17 @@ imap_mbox_list(ImapMboxHandle *handle, const char* what)
 ImapResponse
 imap_mbox_lsub(ImapMboxHandle *handle, const char* what)
 {
+  ImapResponse rc;
   IMAP_REQUIRED_STATE2(handle,IMHS_AUTHENTICATED, IMHS_SELECTED, IMR_BAD);
+  HANDLE_LOCK(handle);
   {
   gchar *mbx7 = imap_utf8_to_mailbox(what);
   gchar *cmd = g_strdup_printf("LSUB \"%s\" \"%%\"", mbx7);
-  ImapResponse rc = imap_cmd_exec(handle, cmd);
+  rc = imap_cmd_exec(handle, cmd);
   g_free(mbx7); g_free(cmd);
-  return rc;
   }
+  HANDLE_UNLOCK(handle);
+  return rc;
 }
 
 
@@ -367,13 +373,15 @@ imap_mbox_status(ImapMboxHandle *r, const char*what,
     gchar *mbx7 = imap_utf8_to_mailbox(what);
     gchar *items = g_strjoinv(" ", (gchar**)&item_arr[0]);
     gchar *cmd = g_strdup_printf("STATUS \"%s\" (%s)", mbx7, items);
+    HANDLE_LOCK(r);
     g_hash_table_insert(r->status_resps, (gpointer)what, res);
     rc = imap_cmd_exec(r, cmd);
     g_hash_table_remove(r->status_resps, what);
+    HANDLE_UNLOCK(r);
     g_free(mbx7); g_free(cmd);
     g_free(items);
   }
-  return IMR_OK; 
+  return rc; 
 }
 /* 6.3.11 APPEND Command */
 static gchar*
@@ -493,7 +501,7 @@ imap_mbox_append_multi_real(ImapMboxHandle *handle,
   if(uid_sequence)
     uid_sequence->ranges = NULL;
 
-  imap_handle_idle_disable(handle);
+  if (!imap_handle_idle_disable(handle)) return IMR_SEVERED;
   for(msg_cnt=0;
       (msg_size = dump_cb(buf, sizeof(buf),
 			  IMA_STAGE_NEW_MSG, &flags, cb_arg)) >0;
@@ -540,7 +548,7 @@ imap_mbox_append_multi_real(ImapMboxHandle *handle,
 	rc = imap_cmd_step (handle, cmdno);
       } while (rc == IMR_UNTAGGED);
       if(rc != IMR_RESPOND) return rc;
-      while( (c=sio_getc(handle->sio)) != -1 && c != '\n');
+      EAT_LINE(handle, c);
       if(c == -1) {
 	imap_handle_disconnect(handle);
 	return IMR_SEVERED;
@@ -668,9 +676,11 @@ imap_mbox_close(ImapMboxHandle *h)
   ImapResponse rc;
   IMAP_REQUIRED_STATE1(h, IMHS_SELECTED, IMR_BAD);
 
+  HANDLE_LOCK(h);
   rc = imap_cmd_exec(h, "CLOSE");  
   if(rc == IMR_OK)
     h->state = IMHS_AUTHENTICATED;
+  HANDLE_UNLOCK(h);
   return rc;
 }
 
@@ -690,11 +700,15 @@ imap_mbox_expunge(ImapMboxHandle *handle)
 ImapResponse
 imap_mbox_expunge_a(ImapMboxHandle *handle)
 {
+  ImapResponse rc;
   IMAP_REQUIRED_STATE2(handle,IMHS_AUTHENTICATED, IMHS_SELECTED, IMR_BAD);
   /* extra care would be required to use this once since no other
      commands that use sequence numbers can be issued before this one
      finishes... */
-  return imap_cmd_issue(handle, "EXPUNGE");
+  HANDLE_LOCK(handle);
+  rc = imap_cmd_issue(handle, "EXPUNGE");
+  HANDLE_UNLOCK(handle);
+  return rc;
 }
 
 /* 6.4.4 SEARCH Command */
@@ -805,7 +819,8 @@ flag_unknown(int i, struct flag_needed_data* d)
 }
   
 /* imap_assure_needed_flags issues several SEARCH queries in one shot
-   and hopes the output is not intermixed */
+   and hopes the output is not intermixed. This functionl must be
+   called with handle locked. */
 ImapResponse
 imap_assure_needed_flags(ImapMboxHandle *h, ImapMsgFlag needed_flags)
 {
@@ -856,7 +871,9 @@ imap_assure_needed_flags(ImapMboxHandle *h, ImapMsgFlag needed_flags)
       case IMSGF_RECENT:   flg = "RECENT"; break;
       default: g_free(seqno); continue;
       }
-      if(!cmd) imap_handle_idle_disable(h);
+      if(!cmd) {
+        if (!imap_handle_idle_disable(h)) { rc = IMR_SEVERED; break; }
+      }
       cmd = g_strdup_printf(cmd_format, seqno, flg);
       g_free(seqno);
       flag[issued_cmd] = fnd.flag;
@@ -894,16 +911,22 @@ imap_mbox_handle_msgno_has_flags(ImapMboxHandle *h, unsigned msgno,
 {
   ImapFlagCache *flags;
   ImapMsgFlag needed_flags;
+  gboolean retval;
 
   IMAP_REQUIRED_STATE1(h, IMHS_SELECTED, IMR_BAD);
+
+  HANDLE_LOCK(h);
   flags = &g_array_index(h->flag_cache, ImapFlagCache, msgno-1);
   
   needed_flags = ~flags->known_flags & (flag_set | flag_unset);
   
-  return 
+  retval =
     (!needed_flags||imap_assure_needed_flags(h, needed_flags) == IMR_OK) &&
     (flags->flag_values & flag_set) == flag_set &&
     (flags->flag_values & flag_unset) == 0;
+  HANDLE_UNLOCK(h);
+
+  return retval;
 }
 
 
@@ -1000,9 +1023,10 @@ imap_mbox_handle_fetch_range(ImapMboxHandle* handle,
   return rc;
 }
 
-ImapResponse
-imap_mbox_handle_fetch_set(ImapMboxHandle* handle,
-                           unsigned *set, unsigned cnt, ImapFetchType ift)
+static ImapResponse
+imap_mbox_handle_fetch_set_unlocked(ImapMboxHandle* handle,
+				    unsigned *set, unsigned cnt,
+				    ImapFetchType ift)
 {
   gchar * seq;
   ImapResponse rc;
@@ -1020,15 +1044,24 @@ imap_mbox_handle_fetch_set(ImapMboxHandle* handle,
   if(seq) {
     const char* hdr[13];
     ic_construct_header_list(hdr, fd.fd.req_fetch_type);
-    HANDLE_LOCK(handle);
     rc = imap_mbox_handle_fetch(handle, seq, hdr);
     if(rc == IMR_OK) set_avail_headers(handle, seq, fd.fd.req_fetch_type);
-    HANDLE_UNLOCK(handle);
     g_free(seq);
   } else rc = IMR_OK;
   return rc;
 }
 
+ImapResponse
+imap_mbox_handle_fetch_set(ImapMboxHandle* handle,
+                           unsigned *set, unsigned cnt, ImapFetchType ift)
+{
+  ImapResponse rc;
+  HANDLE_LOCK(handle);
+  rc = imap_mbox_handle_fetch_set_unlocked(handle, set, cnt, ift);
+  HANDLE_UNLOCK(handle);
+  return rc;
+}
+
 static void
 write_nstring(unsigned seqno, ImapFetchBodyType body_type,
               const char *str, size_t len, void *fl)
@@ -1040,6 +1073,10 @@ write_nstring(unsigned seqno, ImapFetchBodyType body_type,
 struct FetchBodyPassthroughData {
   ImapFetchBodyCb cb;
   void *arg;
+  char *body;
+  size_t body_length;
+  unsigned seqno;
+  unsigned pipeline_error;
 };
 
 static void
@@ -1049,13 +1086,54 @@ fetch_body_passthrough(unsigned seqno,
 		       size_t buflen, void* arg)
 {
   struct FetchBodyPassthroughData* data = (struct FetchBodyPassthroughData*)arg;
-
-  data->cb(seqno, buf, buflen, data->arg);
+  switch(body_type) {
+  case IMAP_BODY_TYPE_RFC822:
+    data->cb(seqno, buf, buflen, data->arg);
+    break;
+  case IMAP_BODY_TYPE_HEADER:
+    if(data->seqno == 0) {
+      data->seqno = seqno;
+      data->cb(seqno, buf, buflen, data->arg);
+    } else {
+      if(data->seqno == seqno) {
+	data->cb(seqno, buf, buflen, data->arg);
+	data->cb(seqno, data->body, data->body_length, data->arg);
+	g_free(data->body);
+	data->body = NULL;
+	data->seqno = 0;
+      } else {
+	/* This server sends data in a strange order that makes
+	   efficient pipeline processing impossible. Just signal an
+	   error. */
+	data->pipeline_error++;
+      }
+    }
+    break;
+  case IMAP_BODY_TYPE_TEXT:
+    if(data->seqno == seqno) {
+      data->cb(seqno, buf, buflen, data->arg);
+      data->seqno = 0;
+    } else {
+      /* Text before header. Still, we can afford to invert it.. */
+      if(data->body)
+	data->pipeline_error++; /* Unlikely... */
+      else {
+	data->body = g_malloc(buflen);
+	memcpy(data->body, buf, buflen);
+	data->body_length = buflen;
+	data->seqno = seqno;
+      }
+    }
+    break;
+  default:
+    data->pipeline_error++;
+  }
 }
 
 ImapResponse
 imap_mbox_handle_fetch_rfc822(ImapMboxHandle* handle,
 			      unsigned cnt, unsigned *set,
+			      gboolean peek_only,
 			      ImapFetchBodyCb fetch_cb,
 			      void *fetch_cb_data)
 {
@@ -1070,10 +1148,16 @@ imap_mbox_handle_fetch_rfc822(ImapMboxHandle* handle,
   if(seq) {
     ImapFetchBodyInternalCb cb = handle->body_cb;
     void                   *arg = handle->body_arg;
-    gchar *cmd = g_strdup_printf("FETCH %s RFC822", seq);
+    gchar *cmd = g_strdup_printf("FETCH %s %s", seq,
+				 peek_only
+				 ? "(BODY.PEEK[HEADER] BODY.PEEK[TEXT])"
+				 : "RFC822");
     struct FetchBodyPassthroughData passthrough_data;
     passthrough_data.cb = fetch_cb;
     passthrough_data.arg = fetch_cb_data;
+    passthrough_data.body = NULL;
+    passthrough_data.seqno = 0;
+    passthrough_data.pipeline_error = 0;
     handle->body_cb  = fetch_cb ? fetch_body_passthrough : NULL;
     handle->body_arg = &passthrough_data;
     rc = imap_cmd_exec(handle, cmd);
@@ -1081,6 +1165,10 @@ imap_mbox_handle_fetch_rfc822(ImapMboxHandle* handle,
     handle->body_arg = arg;
     g_free(cmd);
     g_free(seq);
+    if(passthrough_data.pipeline_error){
+      rc = IMR_NO;
+      imap_mbox_handle_set_msg(handle, "Unordered data received from server");
+    }
   }
   HANDLE_UNLOCK(handle);
 
@@ -1498,16 +1586,19 @@ imap_mbox_unselect(ImapMboxHandle *h)
 ImapResponse
 imap_mbox_thread(ImapMboxHandle *h, const char *how, ImapSearchKey *filter)
 {
+  ImapResponse rc = IMR_NO;
+
   IMAP_REQUIRED_STATE1(h, IMHS_SELECTED, IMR_BAD);
+
+  HANDLE_LOCK(h);
   if(imap_mbox_handle_can_do(h, IMCAP_THREAD_REFERENCES)) {
     int can_do_literals = imap_mbox_handle_can_do(h, IMCAP_LITERAL);
     unsigned cmdno;
-    ImapResponse rc;
     ImapCmdTag tag;
 
     cmdno = imap_make_tag(tag);
     
-    imap_handle_idle_disable(h);
+    if (!imap_handle_idle_disable(h)) { rc = IMR_SEVERED; goto exit_cleanup; }
     sio_printf(h->sio, "%s THREAD %s UTF-8 ", tag, how);
     if(!filter)
       sio_write(h->sio, "ALL", 3);
@@ -1522,9 +1613,11 @@ imap_mbox_thread(ImapMboxHandle *h, const char *how, ImapSearchKey *filter)
       rc = imap_cmd_step(h, cmdno);
     while(rc == IMR_UNTAGGED);
     imap_handle_idle_enable(h, 30);
-    return rc;
-  } else
-    return IMR_NO;
+  }
+ exit_cleanup:
+  HANDLE_UNLOCK(h);
+
+  return rc;
 }
 
 
@@ -1707,8 +1800,8 @@ imap_mbox_sort_msgno_client(ImapMboxHandle *handle, ImapSortKey key,
     qsort(seqno_to_fetch, fetch_cnt, sizeof(unsigned), comp_unsigned);
     printf("Should the client side sorting code "
            "be sorry about your bandwidth usage?\n");
-    rc = imap_mbox_handle_fetch_set(handle, seqno_to_fetch,
-                                    fetch_cnt, fetch_type);
+    rc = imap_mbox_handle_fetch_set_unlocked(handle, seqno_to_fetch,
+					     fetch_cnt, fetch_type);
     if(rc != IMR_OK)
       return rc;
   }
@@ -1746,15 +1839,19 @@ ImapResponse
 imap_mbox_sort_msgno(ImapMboxHandle *handle, ImapSortKey key,
                      int ascending, unsigned int *msgno, unsigned cnt)
 {
+  ImapResponse rc;
   IMAP_REQUIRED_STATE1(handle, IMHS_SELECTED, IMR_BAD);
+
+  HANDLE_LOCK(handle);
   if(imap_mbox_handle_can_do(handle, IMCAP_SORT)) 
-    return imap_mbox_sort_msgno_srv(handle, key, ascending, msgno, cnt);
+    rc = imap_mbox_sort_msgno_srv(handle, key, ascending, msgno, cnt);
   else {
-    return 
-      handle->enable_client_sort
+    rc = handle->enable_client_sort
       ? imap_mbox_sort_msgno_client(handle, key, ascending, msgno, cnt)
       : IMR_NO;
   }
+  HANDLE_UNLOCK(handle);
+  return rc;
 }
 
 static void
@@ -1781,12 +1878,14 @@ imap_mbox_sort_filter(ImapMboxHandle *handle, ImapSortKey key, int ascending,
   unsigned i;
 
   IMAP_REQUIRED_STATE1(handle, IMHS_SELECTED, IMR_BAD);
+
+  HANDLE_LOCK(handle);
   if(key == IMSO_MSGNO) {
     if(filter) { /* CASE 1a */
       handle->mbox_view.entries = 0; /* FIXME: I do not like this! 
                                       * we should not be doing such 
                                       * low level manipulations here */
-      rc = imap_search_exec(handle, FALSE, filter, append_no, NULL);
+      rc = imap_search_exec_unlocked(handle, FALSE, filter, append_no, NULL);
     } else { /* CASE 1b */
       if(handle->thread_root)
         g_node_destroy(handle->thread_root);
@@ -1800,7 +1899,7 @@ imap_mbox_sort_filter(ImapMboxHandle *handle, ImapSortKey key, int ascending,
           g_node_prepend_data(handle->thread_root,
                               GUINT_TO_POINTER(i));
       }
-      return IMR_OK;
+      rc = IMR_OK;
     }
   } else { 
     if(imap_mbox_handle_can_do(handle, IMCAP_SORT)) { /* CASE 2a */
@@ -1811,7 +1910,7 @@ imap_mbox_sort_filter(ImapMboxHandle *handle, ImapSortKey key, int ascending,
 
       cmdno =  imap_make_tag(tag);
       keystr = sort_code_to_string(key);
-      imap_handle_idle_disable(handle);
+      if (!imap_handle_idle_disable(handle)) { rc = IMR_SEVERED; goto cleanup; }
       sio_printf(handle->sio, "%s SORT (%s%s) UTF-8 ", tag,
                  ascending ? "" : "REVERSE ", keystr);
 
@@ -1833,23 +1932,23 @@ imap_mbox_sort_filter(ImapMboxHandle *handle, ImapSortKey key, int ascending,
       while(rc == IMR_UNTAGGED);
     } else {                                           /* CASE 2b */
       /* try client-side sorting... */
-      if(!handle->enable_client_sort)
-        return IMR_NO;
-      handle->mbox_view.entries = 0; /* FIXME: I do not like this! 
-                                      * we should not be doing such 
-                                      * low level manipulations here */
-      if(filter)
-        rc = imap_search_exec(handle, FALSE, filter, append_no, NULL);
-      else {
-        rc = IMR_OK;
-        for(i=0; i<handle->exists; i++)
-          mbox_view_append_no(&handle->mbox_view, i+1);
-      }
-      if(rc != IMR_OK)
-        return rc;
-      rc = imap_mbox_sort_msgno_client(handle, key, ascending,
-                                       handle->mbox_view.arr,
-                                       handle->mbox_view.entries);
+      if(handle->enable_client_sort) {
+	handle->mbox_view.entries = 0; /* FIXME: I do not like this! 
+					* we should not be doing such 
+					* low level manipulations here */
+	if(filter)
+	  rc = imap_search_exec_unlocked(handle, FALSE, filter,
+					 append_no, NULL);
+	else {
+	  rc = IMR_OK;
+	  for(i=0; i<handle->exists; i++)
+	    mbox_view_append_no(&handle->mbox_view, i+1);
+	}
+	if(rc == IMR_OK)
+	  rc = imap_mbox_sort_msgno_client(handle, key, ascending,
+					   handle->mbox_view.arr,
+					   handle->mbox_view.entries);
+      } else rc = IMR_NO;
     }
   }
   if(rc == IMR_OK) {
@@ -1861,6 +1960,9 @@ imap_mbox_sort_filter(ImapMboxHandle *handle, ImapSortKey key, int ascending,
       g_node_append_data(handle->thread_root,
                          GUINT_TO_POINTER(handle->mbox_view.arr[i]));
   }
+ cleanup:
+  HANDLE_UNLOCK(handle);
+
   return rc;
 }
 
@@ -1879,8 +1981,8 @@ imap_mbox_filter_msgnos(ImapMboxHandle *handle, ImapSearchKey *filter,
 {
   ImapResponse res;
   HANDLE_LOCK(handle);
-  res= imap_search_exec(handle, FALSE, filter,
-			(ImapSearchCb)make_msgno_table, msgnos);
+  res = imap_search_exec_unlocked(handle, FALSE, filter,
+				  (ImapSearchCb)make_msgno_table, msgnos);
   HANDLE_UNLOCK(handle);
   return res;
 }
