@@ -27,6 +27,7 @@
 #include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #if defined(USE_TLS)
 #include <openssl/ssl.h>
@@ -390,7 +391,7 @@ pop_authenticate(PopHandle *pop, const char *greeting, GError **err)
   const PopAuthenticator *authenticator;
   ImapResult r;
 
-  if(g_strncasecmp(greeting, "+OK", 3) != 0) {
+  if(g_ascii_strncasecmp(greeting, "+OK", 3) != 0) {
     g_set_error(err, IMAP_ERROR, IMAP_POP_AUTH_ERROR,
                 "Server does not welcome us: %s", greeting);
     return FALSE;
@@ -608,7 +609,7 @@ pop_get_uid(PopHandle *pop, unsigned msgno, GError **err)
     while( sio_gets(pop->sio, line, sizeof(line)) &&
            strcmp(line, ".\r\n") ) {
       char* cr, *space = strchr(line, ' ');
-      unsigned read_msgno = atoi(line);
+      unsigned read_msgno = strtol(line, NULL, 10);
       if(!space ||read_msgno != curr_msgno) /* Parsing error? */
         continue;
 
