@@ -20,9 +20,11 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
-
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 #include "information.h"
+
 #ifdef HAVE_NOTIFY
 #include <libnotify/notify.h>
 #include <gtk/gtk.h>
