@@ -20,69 +20,12 @@
  * 02111-1307, USA.
  */
 
-#include <string.h>
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 #include "libbalsa-conf.h"
 
-#if !GLIB_CHECK_VERSION(2, 6, 0)
-
-#define BALSA_CONFIG_PREFIX "balsa/"
-
-/* 
- * Call @func for each section name that begins with @prefix.
- * @func is called with arguments:
- *   const gchar * @key		the section;
- *   const gchar * @value	the trailing part of the section name,
- *   				following the @prefix;
- *   gpointer @data		the @data passed in.
- * Iteration terminates when @func returns TRUE.
- */
-void
-libbalsa_conf_foreach_group(const gchar * prefix,
-                            LibBalsaConfForeachFunc func, gpointer data)
-{
-    gsize pref_len;
-    void *iterator;
-    gchar *key;
-
-    pref_len = strlen(prefix);
-    iterator = gnome_config_init_iterator_sections(BALSA_CONFIG_PREFIX);
-    while ((iterator = gnome_config_iterator_next(iterator, &key, NULL))) {
-        if (strncmp(key, prefix, pref_len) == 0
-            && func(key, key + pref_len, data)) {
-            g_free(key);
-            g_free(iterator);
-            break;
-        }
-        g_free(key);
-    }
-}
-
-void
-libbalsa_conf_push_group(const char *group)
-{
-    gchar *prefix = g_strconcat(BALSA_CONFIG_PREFIX, group, "/", NULL);
-    gnome_config_push_prefix(prefix);
-    g_free(prefix);
-}
-
-void
-libbalsa_conf_remove_group_(const char *group, gboolean priv)
-{
-    gchar *prefix = g_strconcat(BALSA_CONFIG_PREFIX, group, "/", NULL);
-    gnome_config_clean_section_(prefix, priv);
-    g_free(prefix);
-}
-
-gboolean
-libbalsa_conf_has_group(const char *group)
-{
-    gchar *prefix = g_strconcat(BALSA_CONFIG_PREFIX, group, "/", NULL);
-    gboolean retval = gnome_config_has_section(prefix);
-    g_free(prefix);
-    return retval;
-}
-
-#else                           /* !GLIB_CHECK_VERSION(2, 6, 0) */
+#include <string.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -93,14 +36,11 @@ libbalsa_conf_has_group(const char *group)
 
 #include "libbalsa.h"
 #include "misc.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 typedef struct {
     GKeyFile *key_file;
     gchar *path;
-#if !GLIB_CHECK_VERSION(2, 7, 0)
-    gchar *new_path;
-#endif                          /* !GLIB_CHECK_VERSION(2, 7, 0) */
     guint changes;
     time_t mtime;
 } LibBalsaConf;
@@ -142,7 +82,7 @@ lbc_init(LibBalsaConf * conf, const gchar * filename,
          const gchar * old_dir)
 {
     struct stat buf;
-    GError *error;
+    GError *error = NULL;
 
     if (!conf->path)
         conf->path =
@@ -154,11 +94,7 @@ lbc_init(LibBalsaConf * conf, const gchar * filename,
     } else
         conf->key_file = g_key_file_new();
 
-#if !GLIB_CHECK_VERSION(2, 7, 0)
-    conf->new_path = g_strconcat(conf->path, ".new", NULL);
-#endif                          /* !GLIB_CHECK_VERSION(2, 7, 0) */
     libbalsa_assure_balsa_dir();
-    error = NULL;
     if (!g_key_file_load_from_file
         (conf->key_file, conf->path, G_KEY_FILE_NONE, &error)) {
         gchar *old_path;
@@ -171,8 +107,7 @@ lbc_init(LibBalsaConf * conf, const gchar * filename,
         g_message("Could not load config from \"%s\":\n %s\n"
                   " trying \"%s\"", conf->path, error->message, old_path);
 #endif                          /* DEBUG */
-        g_error_free(error);
-        error = NULL;
+        g_clear_error(&error);
 
         buf = lbc_readfile(old_path);
         if (buf) {
@@ -191,20 +126,15 @@ lbc_init(LibBalsaConf * conf, const gchar * filename,
                       old_path,
                       error ? error->message : g_strerror(errno));
 #endif                          /* DEBUG */
-            if (error) {
-                g_error_free(error);
-                error = NULL;
-            }
+            g_clear_error(&error);
             warn = FALSE;
         }
         g_free(old_path);
-        if (warn) {
+        if (warn)
             libbalsa_information(LIBBALSA_INFORMATION_WARNING,
                                  _("Your Balsa configuration "
                                    "is now stored in "
                                    "\"~/.balsa/config\"."));
-            warn = FALSE;
-        }
     }
 }
 
@@ -264,6 +194,29 @@ libbalsa_conf_foreach_group(const gchar * prefix,
     lbc_unlock();
 }
 
+void
+libbalsa_conf_foreach_keys(const gchar * group,
+			   LibBalsaConfForeachFunc func, gpointer data)
+{
+    gchar **keys, **key;
+
+    lbc_lock();
+
+    if ((keys = g_key_file_get_keys(lbc_conf.key_file, group, NULL, NULL))) {
+	for (key = keys; *key; key++) {
+	    gchar * val = g_key_file_get_value(lbc_conf.key_file, group, *key, NULL);
+	    if (func(*key, val, data)) {
+		g_free(val);
+		break;
+	    }
+	    g_free(val);
+	}
+	g_strfreev(keys);
+    }
+
+    lbc_unlock();
+}
+
 void
 libbalsa_conf_push_group(const gchar * group)
 {
@@ -391,6 +344,32 @@ libbalsa_conf_get_int_with_default_(const char *path,
     return retval;
 }
 
+gdouble
+libbalsa_conf_get_double_with_default_(const char *path,
+				       gboolean * def, gboolean priv)
+{
+    gchar *key;
+    const gchar *defval;
+    gdouble retval;
+    GError *error = NULL;
+
+    key = lbc_get_key(path, &defval);
+    retval =
+        g_key_file_get_double(LBC_KEY_FILE(priv), lbc_groups->data, key,
+			      &error);
+    g_free(key);
+    if (error) {
+        g_error_free(error);
+        if (defval)
+            retval = g_ascii_strtod(defval, NULL);
+    }
+
+    if (def)
+        *def = error != NULL;
+
+    return retval;
+}
+
 void
 libbalsa_conf_set_int_(const char *path, int value, gboolean priv)
 {
@@ -399,6 +378,14 @@ libbalsa_conf_set_int_(const char *path, int value, gboolean priv)
     LBC_CHANGED(priv);
 }
 
+void
+libbalsa_conf_set_double_(const char *path, double value, gboolean priv)
+{
+    g_key_file_set_double(LBC_KEY_FILE(priv), lbc_groups->data, path,
+			  value);
+    LBC_CHANGED(priv);
+}
+
 void
 libbalsa_conf_set_string_(const char *path, const char *value,
                           gboolean priv)
@@ -467,10 +454,6 @@ lbc_drop_all(LibBalsaConf * conf)
     conf->key_file = NULL;
     g_free(conf->path);
     conf->path = NULL;
-#if !GLIB_CHECK_VERSION(2, 7, 0)
-    g_free(conf->new_path);
-    conf->new_path = NULL;
-#endif                          /* !GLIB_CHECK_VERSION(2, 7, 0) */
     conf->changes = 0;
 }
 
@@ -489,10 +472,7 @@ lbc_sync(LibBalsaConf * conf)
     gchar **groups;
     gchar *buf;
     gsize len;
-    GError *error;
-#if !GLIB_CHECK_VERSION(2, 7, 0)
-    gint fd;
-#endif                          /* !GLIB_CHECK_VERSION(2, 7, 0) */
+    GError *error = NULL;
 
     if (!conf->changes)
         return;
@@ -506,7 +486,6 @@ lbc_sync(LibBalsaConf * conf)
     }
     g_strfreev(groups);
 
-    error = NULL;
     buf = g_key_file_to_data(conf->key_file, &len, &error);
     if (error) {
 #if DEBUG
@@ -518,7 +497,6 @@ lbc_sync(LibBalsaConf * conf)
         return;
     }
 
-#if GLIB_CHECK_VERSION(2, 7, 0)
     if (!g_file_set_contents(conf->path, buf, len, &error)) {
         if (error) {
 #if DEBUG
@@ -535,50 +513,6 @@ lbc_sync(LibBalsaConf * conf)
     }
 
     g_free(buf);
-#else                           /* GLIB_CHECK_VERSION(2, 7, 0) */
-    fd = g_open(conf->new_path, O_WRONLY | O_CREAT | O_TRUNC,
-                S_IRUSR | S_IWUSR);
-    if (fd < 0) {
-#if DEBUG
-        g_message("Failed to open temporary config file \"%s\"\n"
-                  " changes not saved", conf->new_path);
-#endif                          /* DEBUG */
-        g_free(buf);
-        return;
-    }
-
-    if (write(fd, buf, len) < (gssize) len) {
-#if DEBUG
-        g_message("Failed to write temporary config file \"%s\"\n"
-                  " changes not saved", conf->new_path);
-#endif                          /* DEBUG */
-        close(fd);
-        g_unlink(conf->new_path);
-        g_free(buf);
-        return;
-    }
-    close(fd);
-    g_free(buf);
-
-    if (g_unlink(conf->path) < 0
-        && g_file_error_from_errno(errno) != G_FILE_ERROR_NOENT) {
-#if DEBUG
-        g_message("Failed to unlink config file \"%s\"\n"
-                  " new config file saved as \"%s\"", conf->path,
-                  conf->new_path);
-        perror("Config");
-#endif                          /* DEBUG */
-        return;
-    }
-
-    if (g_rename(conf->new_path, conf->path) < 0) {
-#if DEBUG
-        g_message("Failed to rename temporary config file \"%s\"\n",
-                  conf->new_path);
-#endif                          /* DEBUG */
-        return;
-    }
-#endif                          /* GLIB_CHECK_VERSION(2, 7, 0) */
 }
 
 void
@@ -589,5 +523,3 @@ libbalsa_conf_sync(void)
     lbc_sync(&lbc_conf_priv);
     lbc_unlock();
 }
-
-#endif                          /* !GLIB_CHECK_VERSION(2, 6, 0) */
