@@ -20,9 +20,10 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
-
-#define _XOPEN_SOURCE 500
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "libbalsa.h"
 
 #include <glib.h>
 
@@ -60,8 +61,8 @@
 #endif
 #endif
 
-#include "libbalsa.h"
 #include "misc.h"
+#include "missing.h"
 #include <glib/gi18n.h>
 
 #ifdef BALSA_USE_THREADS
@@ -101,7 +102,7 @@ libbalsa_init(LibBalsaInformationFunc information_callback)
 
     libbalsa_real_information_func = information_callback;
 
-    g_mime_init(0);
+    g_mime_init(GMIME_ENABLE_RFC2047_WORKAROUNDS);
 
     GMIME_TYPE_DATA_WRAPPER;
     GMIME_TYPE_FILTER;
@@ -639,7 +640,12 @@ gboolean
 libbalsa_abort_on_timeout(const char *host)
 {  /* It appears not to be entirely thread safe... Some locks do not
       get released as they should be. */
-    return libbalsa_ask(ask_timeout_real, (void*)host) != 0; 
+    char *hostname;
+
+    hostname = g_alloca (strlen (host) + 1);
+    strcpy (hostname, host);
+    
+    return libbalsa_ask(ask_timeout_real, hostname) != 0; 
 }
 
 
@@ -846,9 +852,9 @@ libbalsa_get_image_from_face_header(const gchar * content, GError ** err)
     GtkWidget *image = NULL;
 
     stream = g_mime_stream_mem_new();
-    stream_filter = g_mime_stream_filter_new_with_stream(stream);
+    stream_filter = g_mime_stream_filter_new(stream);
 
-    filter = g_mime_filter_basic_new_type(GMIME_FILTER_BASIC_BASE64_DEC);
+    filter = g_mime_filter_basic_new(GMIME_CONTENT_ENCODING_BASE64, FALSE);
     g_mime_stream_filter_add(GMIME_STREAM_FILTER(stream_filter), filter);
     g_object_unref(filter);
 
