@@ -23,6 +23,12 @@
 #ifndef __LIBBALSA_H__
 #define __LIBBALSA_H__
 
+#ifndef BALSA_VERSION
+# error "Include config.h before this file."
+#endif
+
+typedef struct _LibBalsaCondition LibBalsaCondition;
+typedef struct _LibBalsaIdentity LibBalsaIdentity;
 typedef struct _LibBalsaMailbox LibBalsaMailbox;
 typedef struct _LibBalsaMailboxRemote LibBalsaMailboxRemote;
 typedef struct _LibBalsaMailboxSearchIter LibBalsaMailboxSearchIter;
@@ -30,8 +36,10 @@ typedef struct _LibBalsaMessage LibBalsaMessage;
 typedef struct _LibBalsaMessageHeaders LibBalsaMessageHeaders;
 typedef struct _LibBalsaMessageBody LibBalsaMessageBody;
 typedef struct _LibBalsaServer LibBalsaServer;
+#if ENABLE_ESMTP
 typedef struct _LibBalsaSmtpServer LibBalsaSmtpServer;
-typedef struct _LibBalsaCondition LibBalsaCondition;
+#endif                          /* ENABLE_ESMTP */
+typedef struct _LibbalsaVfs LibbalsaVfs;
 
 
 #include "message.h"
