@@ -22,59 +22,60 @@
 #ifndef __LIBBALSA_MAILBOX_IMAP_H__
 #define __LIBBALSA_MAILBOX_IMAP_H__
 
-#define LIBBALSA_TYPE_MAILBOX_IMAP			(libbalsa_mailbox_imap_get_type())
-#define LIBBALSA_MAILBOX_IMAP(obj)			(GTK_CHECK_CAST (obj, LIBBALSA_TYPE_MAILBOX_IMAP, LibBalsaMailboxImap))
-#define LIBBALSA_MAILBOX_IMAP_CLASS(klass)		(GTK_CHECK_CLASS_CAST (klass, LIBBALSA_TYPE_MAILBOX_IMAP, LibBalsaMailboxImapClass))
-#define LIBBALSA_IS_MAILBOX_IMAP(obj)		(GTK_CHECK_TYPE (obj, LIBBALSA_TYPE_MAILBOX_IMAP))
-#define LIBBALSA_IS_MAILBOX_IMAP_CLASS(klass)	(GTK_CHECK_CLASS_TYPE (klass, LIBBALSA_TYPE_MAILBOX_IMAP))
+#define LIBBALSA_TYPE_MAILBOX_IMAP \
+    (libbalsa_mailbox_imap_get_type())
+#define LIBBALSA_MAILBOX_IMAP(obj) \
+    (G_TYPE_CHECK_INSTANCE_CAST ((obj), LIBBALSA_TYPE_MAILBOX_IMAP, \
+                                 LibBalsaMailboxImap))
+#define LIBBALSA_MAILBOX_IMAP_CLASS(klass) \
+    (G_TYPE_CHECK_CLASS_CAST ((klass), LIBBALSA_TYPE_MAILBOX_IMAP, \
+                              LibBalsaMailboxImapClass))
+#define LIBBALSA_IS_MAILBOX_IMAP(obj) \
+    (G_TYPE_CHECK_INSTANCE_TYPE ((obj), LIBBALSA_TYPE_MAILBOX_IMAP))
+#define LIBBALSA_IS_MAILBOX_IMAP_CLASS(klass) \
+    (G_TYPE_CHECK_CLASS_TYPE ((klass), LIBBALSA_TYPE_MAILBOX_IMAP))
 
-GtkType libbalsa_mailbox_imap_get_type(void);
+GType libbalsa_mailbox_imap_get_type(void);
 
 typedef struct _LibBalsaMailboxImap LibBalsaMailboxImap;
 typedef struct _LibBalsaMailboxImapClass LibBalsaMailboxImapClass;
 
-enum _ImapAuthType {
-    AuthLogin,
-    AuthCram,
-    AuthGSS
-};
-typedef enum _ImapAuthType ImapAuthType;
-typedef guint32 ImapUID;
 #define POINTER_TO_UID(p) GPOINTER_TO_UINT(p)
 #define UID_TO_POINTER(p) GUINT_TO_POINTER(p)
 
-struct _LibBalsaMailboxImap {
-    LibBalsaMailboxRemote mailbox;
-
-    gchar *path;		/* Imap local path (third part of URL) */
-    ImapAuthType auth_type;	/* accepted authentication type */
-    ImapUID      uid_validity;
-};
-
-struct _LibBalsaMailboxImapClass {
-    LibBalsaMailboxRemoteClass klass;
-};
-
-GtkObject *libbalsa_mailbox_imap_new(void);
+LibBalsaMailboxImap *libbalsa_mailbox_imap_new(void);
 
 void libbalsa_mailbox_imap_update_url(LibBalsaMailboxImap* mailbox);
 void libbalsa_mailbox_imap_set_path(LibBalsaMailboxImap * mailbox,
 				    const gchar * path);
+const gchar* libbalsa_mailbox_imap_get_path(LibBalsaMailboxImap * mailbox);
 
 gboolean libbalsa_mailbox_imap_subscribe(LibBalsaMailboxImap * mailbox, 
                                          gboolean subscribe);
 
+GHashTable * libbalsa_mailbox_imap_get_matchings(LibBalsaMailboxImap* mbox,
+						 LibBalsaCondition *condition,
+						 gboolean only_recent,
+						 gboolean * err);
+
+void libbalsa_mailbox_imap_noop(LibBalsaMailboxImap* mbox);
+
 void libbalsa_imap_close_all_connections(void);
 
-void libbalsa_imap_new_subfolder(const gchar * parent, const gchar * folder,
-				 gboolean subscribe, LibBalsaServer * server);
+gboolean libbalsa_imap_new_subfolder(const gchar *parent, const gchar *folder,
+                                     gboolean subscribe,
+                                     LibBalsaServer *server, GError **err);
 
 gboolean libbalsa_imap_rename_subfolder(LibBalsaMailboxImap* mbox,
                                         const gchar *new_parent,
-                                        const gchar *folder, gboolean subscr);
+                                        const gchar *folder, gboolean subscr,
+                                        GError **err);
 
-void libbalsa_imap_delete_folder(LibBalsaMailboxImap * mailbox);
+gboolean libbalsa_imap_delete_folder(LibBalsaMailboxImap * mailbox,
+                                     GError **err);
 
-void reset_mutt_passwords(LibBalsaServer *);
+gchar *libbalsa_imap_url(LibBalsaServer * server, const gchar * path);
 
+void libbalsa_imap_set_cache_size(off_t cache_size);
+void libbalsa_imap_purge_temp_dir(off_t cache_size);
 #endif				/* __LIBBALSA_MAILBOX_IMAP_H__ */
