@@ -20,7 +20,9 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -437,11 +439,7 @@ libbalsa_mailbox_local_load_config(LibBalsaMailbox * mailbox,
  */
 
 typedef struct {
-#if GLIB_CHECK_VERSION(2, 8, 0)
     GArray * array;
-#else                           /* GLIB_CHECK_VERSION(2, 8, 0) */
-    int fd;
-#endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
     guint (*fileno)(LibBalsaMailboxLocal * local, guint msgno);
     LibBalsaMailboxLocal *local;
 } LibBalsaMailboxLocalSaveTreeInfo;
@@ -474,11 +472,7 @@ lbm_local_save_tree_item(guint msgno, guint a,
         info.value.parent = a;
     }
 
-#if GLIB_CHECK_VERSION(2, 8, 0)
     return g_array_append_val(save_info->array, info) == NULL;
-#else                           /* GLIB_CHECK_VERSION(2, 8, 0) */
-    return write(save_info->fd, &info, sizeof info) != sizeof info;
-#endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
 }
 
 static gboolean
@@ -512,11 +506,7 @@ lbm_local_save_tree(LibBalsaMailboxLocal * local)
     LibBalsaMailbox *mailbox = LIBBALSA_MAILBOX(local);
     gchar *filename;
     LibBalsaMailboxLocalSaveTreeInfo save_info;
-#if GLIB_CHECK_VERSION(2, 8, 0)
-    GError *err;
-#else                           /* GLIB_CHECK_VERSION(2, 8, 0) */
-    gchar *template;
-#endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
+    GError *err = NULL;
 
     if (!mailbox->msg_tree || !mailbox->msg_tree_changed)
         return;
@@ -536,33 +526,16 @@ lbm_local_save_tree(LibBalsaMailboxLocal * local)
 
     save_info.fileno = LIBBALSA_MAILBOX_LOCAL_GET_CLASS(local)->fileno;
     save_info.local = local;
-#if GLIB_CHECK_VERSION(2, 8, 0)
     save_info.array =
         g_array_new(FALSE, FALSE, sizeof(LibBalsaMailboxLocalTreeInfo));
     lbm_local_save_tree_item(0, libbalsa_mailbox_get_total(mailbox),
                              &save_info);
-#else                           /* GLIB_CHECK_VERSION(2, 8, 0) */
-    template = g_strconcat(filename, ":XXXXXX", NULL);
-    save_info.fd = g_mkstemp(template);
-    if (save_info.fd < 0
-        || lbm_local_save_tree_item(0, libbalsa_mailbox_get_total(mailbox),
-                                    &save_info)) {
-        libbalsa_information(LIBBALSA_INFORMATION_WARNING,
-                             _("Failed to create temporary file \"%s\": %s"),
-                             template, strerror(errno));
-        g_free(template);
-        g_free(filename);
-        return;
-    }
-#endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
 
     /* Pre-order is required for the file to be created correctly. */
     g_node_traverse(mailbox->msg_tree, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
                     (GNodeTraverseFunc) lbm_local_save_tree_func,
                     &save_info);
 
-#if GLIB_CHECK_VERSION(2, 8, 0)
-    err = NULL;
     if (!g_file_set_contents(filename, save_info.array->data,
                              save_info.array->len *
                              sizeof(LibBalsaMailboxLocalTreeInfo), &err)) {
@@ -572,17 +545,6 @@ lbm_local_save_tree(LibBalsaMailboxLocal * local)
         g_error_free(err);
     }
     g_array_free(save_info.array, TRUE);
-#else                           /* GLIB_CHECK_VERSION(2, 8, 0) */
-    if (close(save_info.fd) != 0
-        || (unlink(filename) != 0 && errno != ENOENT)
-        || libbalsa_safe_rename(template, filename) != 0)
-        libbalsa_information(LIBBALSA_INFORMATION_WARNING,
-                             _("Failed to save cache file \"%s\": %s.  "
-                               "New version saved as \"%s\""),
-                             filename, strerror(errno), template);
-
-    g_free(template);
-#endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
     g_free(filename);
 }
 
@@ -592,6 +554,7 @@ lbm_local_restore_tree(LibBalsaMailboxLocal * local, guint * total)
     LibBalsaMailbox *mailbox = LIBBALSA_MAILBOX(local);
     gchar *filename;
     gchar *name;
+    struct stat st;
     gchar *contents;
     gsize length;
     GError *err = NULL;
@@ -603,7 +566,8 @@ lbm_local_restore_tree(LibBalsaMailboxLocal * local, guint * total)
     name = mailbox->name ? g_strdup(mailbox->name) :
         g_path_get_basename(libbalsa_mailbox_local_get_path(local));
 
-    if (!g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
+    if (stat(filename, &st) < 0
+        || st.st_mtime < libbalsa_mailbox_get_mtime(mailbox)) {
         /* No error, but we return FALSE so the caller can grab all the
          * message info needed to rethread from scratch. */
         if (libbalsa_mailbox_total_messages(mailbox) > 0)
@@ -1035,8 +999,13 @@ libbalsa_mailbox_local_cache_message(LibBalsaMailboxLocal * local,
     info->message_id = g_strdup(message->message_id);
     info->refs_for_threading =
         libbalsa_message_refs_for_threading(message);
-    info->sender =
-        internet_address_list_to_string(message->headers->from, FALSE);
+
+    info->sender = NULL;
+    if (message->headers->from)
+        info->sender =
+            internet_address_list_to_string(message->headers->from, FALSE);
+    if (!info->sender)
+        info->sender = g_strdup("");
 }
 
 void
