@@ -20,9 +20,10 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 
-#define _XOPEN_SOURCE 500
 
 #include <gmime/gmime-stream-fs.h>
 
@@ -44,6 +45,8 @@
 /* for mx_lock_file and mx_unlock_file */
 #include "mailbackend.h"
 #include "mime-stream-shared.h"
+#include "missing.h"
+
 #include <glib/gi18n.h>
 
 /* #define DEBUG_SEEK TRUE */
@@ -353,7 +356,7 @@ static void mbox_unlock(LibBalsaMailbox * mailbox, GMimeStream *stream)
  */
 static void
 lbm_mbox_header_cb(GMimeParser * parser, const char *header,
-                   const char *value, off_t offset,
+                   const char *value, gint64 offset,
                    gpointer user_data)
 {
     struct message_info *msg_info = *(struct message_info **) user_data;
@@ -391,9 +394,9 @@ static void
 lbm_mbox_save(LibBalsaMailboxMbox * mbox)
 {
     gchar *filename;
-#if GLIB_CHECK_VERSION(2, 8, 0) && !defined(__APPLE__)
+#if !defined(__APPLE__)
     GError *err = NULL;
-#endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
+#endif                          /* !defined(__APPLE__) */
 
     if (!mbox->messages_info_changed)
         return;
@@ -407,10 +410,10 @@ lbm_mbox_save(LibBalsaMailboxMbox * mbox)
             g_array_sized_new(FALSE, FALSE, sizeof(struct message_info), 
                               mbox->msgno_2_msg_info->len);
         guint msgno;
-#if !GLIB_CHECK_VERSION(2, 8, 0) || defined(__APPLE__)
+#if defined(__APPLE__)
         gchar *template;
         gint fd;
-#endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
+#endif                          /* !defined(__APPLE__) */
 
         for (msgno = 1; msgno <= mbox->msgno_2_msg_info->len; msgno++) {
             struct message_info *msg_info =
@@ -418,7 +421,7 @@ lbm_mbox_save(LibBalsaMailboxMbox * mbox)
             g_array_append_val(messages_info, *msg_info);
         }
 
-#if GLIB_CHECK_VERSION(2, 8, 0) && !defined(__APPLE__)
+#if !defined(__APPLE__)
         if (!g_file_set_contents(filename, messages_info->data,
                                  messages_info->len
                                  * sizeof(struct message_info), &err)) {
@@ -427,7 +430,7 @@ lbm_mbox_save(LibBalsaMailboxMbox * mbox)
                                  filename, err->message);
             g_error_free(err);
         }
-#else                           /* GLIB_CHECK_VERSION(2, 8, 0) */
+#else                           /* !defined(__APPLE__) */
         template = g_strconcat(filename, ":XXXXXX", NULL);
         fd = g_mkstemp(template);
         if (fd < 0 || write(fd, messages_info->data,
@@ -451,7 +454,7 @@ lbm_mbox_save(LibBalsaMailboxMbox * mbox)
                                    "New version saved as \"%s\""),
                                  filename, strerror(errno), template);
         g_free(template);
-#endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
+#endif                          /* !defined(__APPLE__) */
         g_array_free(messages_info, TRUE);
     } else if (unlink(filename) < 0)
         libbalsa_information(LIBBALSA_INFORMATION_WARNING,
@@ -581,20 +584,23 @@ static void
 lbm_mbox_restore(LibBalsaMailboxMbox * mbox)
 {
     gchar *filename;
+    struct stat st;
     gchar *contents;
     gsize length;
-    gboolean tmp;
     off_t end;
     struct message_info *msg_info;
     GMimeStream *mbox_stream;
 
     filename = lbm_mbox_get_cache_filename(mbox);
-    tmp = g_file_get_contents(filename, &contents, &length, NULL);
-    g_free(filename);
-
-    if (!tmp)
-        /* Nonexistent file is no error. */
+    if (stat(filename, &st) < 0 
+        || st.st_mtime < libbalsa_mailbox_get_mtime(LIBBALSA_MAILBOX(mbox))
+        || !g_file_get_contents(filename, &contents, &length, NULL)) {
+        /* No cache file, stale cache, or read error. */
+        g_free(filename);
         return;
+    }
+
+    g_free(filename);
 
     if (length < sizeof(struct message_info)) {
         /* Error: file always contains at least one record. */
@@ -1694,10 +1700,12 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	}
 	if (msg_info->local_info.message)
 	    msg_info->local_info.message->msgno = j + 1;
-	j++;
 
 	msg_info->status = msg_info->x_status = msg_info->mime_version = -1;
 	mime_msg = g_mime_parser_construct_message(gmime_parser);
+        if (!mime_msg)
+            /* Try to recover */
+            continue;
         msg_info->start = g_mime_parser_get_from_offset(gmime_parser);
 
 	/* Make sure we don't have offsets for any encapsulated headers. */
@@ -1709,11 +1717,16 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	    msg_info->mime_version = -1;
 
 	from = g_mime_parser_get_from(gmime_parser);
+        if (!from) {
+            /* Try to recover */
+            g_object_unref(mime_msg);
+            continue;
+        }
+
         msg_info->from_len = strlen(from) + 1;
         g_free(from);
 	msg_info->end = g_mime_parser_tell(gmime_parser);
 	msg_info->orig_flags = REAL_FLAGS(msg_info->local_info.flags);
-	g_assert(mime_msg != NULL);
 	g_assert(mime_msg->mime_part != NULL);
 	if (!msg_info->local_info.message || !msg_info->local_info.message->mime_msg)
 	    g_object_unref(mime_msg);
@@ -1726,6 +1739,8 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	    libbalsa_message_body_set_mime_body(msg_info->local_info.message->body_list,
 						mime_msg->mime_part);
 	}
+
+	j++;
     }
     libbalsa_mime_stream_shared_unlock(mbox_stream);
     mbox->msgno_2_msg_info->len = j;
@@ -1787,7 +1802,7 @@ lbm_mbox_message_new(GMimeMessage * mime_message,
 
     message = libbalsa_message_new();
 
-    header = g_mime_message_get_header (mime_message, "Status");
+    header = g_mime_object_get_header (GMIME_OBJECT(mime_message), "Status");
     if (header) {
 	if (strchr(header, 'R') == NULL) /* not found == not READ */
 	    flags |= LIBBALSA_MESSAGE_FLAG_NEW;
@@ -1797,7 +1812,7 @@ lbm_mbox_message_new(GMimeMessage * mime_message,
 	    flags |= LIBBALSA_MESSAGE_FLAG_RECENT;
     } else
 	    flags |= LIBBALSA_MESSAGE_FLAG_NEW |  LIBBALSA_MESSAGE_FLAG_RECENT;
-    header = g_mime_message_get_header (mime_message, "X-Status");
+    header = g_mime_object_get_header (GMIME_OBJECT(mime_message), "X-Status");
     if (header) {
 	if (strchr(header, 'D') != NULL) /* found == DELETED */
 	    flags |= LIBBALSA_MESSAGE_FLAG_DELETED;
@@ -1821,10 +1836,10 @@ static void update_message_status_headers(GMimeMessage *message,
     /* Create headers with spaces in place of flags, if necessary, so we
      * can later update them in place. */
     lbm_mbox_status_hdr(flags, 2, new_header);
-    g_mime_message_set_header(message, "Status", new_header->str);
+    g_mime_object_set_header(GMIME_OBJECT(message), "Status", new_header->str);
     g_string_truncate(new_header, 0);
     lbm_mbox_x_status_hdr(flags, 3, new_header);
-    g_mime_message_set_header(message, "X-Status", new_header->str);
+    g_mime_object_set_header(GMIME_OBJECT(message), "X-Status", new_header->str);
     g_string_free(new_header, TRUE);
 }
 
@@ -1832,7 +1847,8 @@ static void update_message_status_headers(GMimeMessage *message,
  * Encode text parts as quoted-printable.
  */
 static void
-lbm_mbox_prepare_object(GMimeObject * mime_part)
+lbm_mbox_prepare_object(GMimeObject * parent, GMimeObject * mime_part,
+                        gpointer data)
 {
     g_mime_object_remove_header(mime_part, "Content-Length");
 
@@ -1842,21 +1858,30 @@ lbm_mbox_prepare_object(GMimeObject * mime_part)
             /* Do not break crypto. */
             return;
 
+        if (parent)
+            /* We have been called by g_mime_multipart_foreach, which
+             * recursively descends the multipart tree, so we must not
+             * start another descent. */
+            return;
+
         g_mime_multipart_foreach((GMimeMultipart *) mime_part,
-                                 (GMimePartFunc) lbm_mbox_prepare_object,
+                                 lbm_mbox_prepare_object,
                                  NULL);
     } else if (GMIME_IS_MESSAGE_PART(mime_part))
-        lbm_mbox_prepare_object(GMIME_OBJECT
+        lbm_mbox_prepare_object(NULL, 
+                                GMIME_OBJECT
                                 (((GMimeMessagePart *) mime_part)->
-                                 message));
+                                 message), NULL);
     else if (GMIME_IS_MESSAGE(mime_part))
-        lbm_mbox_prepare_object(((GMimeMessage *) mime_part)->mime_part);
+        lbm_mbox_prepare_object(NULL, 
+                                ((GMimeMessage *) mime_part)->mime_part,
+                                NULL);
     else if (!GMIME_IS_MESSAGE_PARTIAL(mime_part)) {
-        GMimePartEncodingType encoding;
-        const GMimeContentType *mime_type;
+        GMimeContentEncoding encoding;
+        GMimeContentType *mime_type;
 
-        encoding = g_mime_part_get_encoding(GMIME_PART(mime_part));
-        if (encoding == GMIME_PART_ENCODING_BASE64)
+        encoding = g_mime_part_get_content_encoding(GMIME_PART(mime_part));
+        if (encoding == GMIME_CONTENT_ENCODING_BASE64)
             return;
 
         mime_type = g_mime_object_get_content_type(mime_part);
@@ -1868,8 +1893,8 @@ lbm_mbox_prepare_object(GMimeObject * mime_part)
                 return;
         }
 
-        g_mime_part_set_encoding(GMIME_PART(mime_part),
-                                 GMIME_PART_ENCODING_QUOTEDPRINTABLE);
+        g_mime_part_set_content_encoding(GMIME_PART(mime_part),
+                                 GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE);
     }
 }
 
@@ -1882,7 +1907,7 @@ lbm_mbox_armored_object(GMimeStream * stream)
     parser = g_mime_parser_new_with_stream(stream);
     object = GMIME_OBJECT(g_mime_parser_construct_message(parser));
     g_object_unref(parser);
-    lbm_mbox_prepare_object(object);
+    lbm_mbox_prepare_object(NULL, object, NULL);
 
     return object;
 }
@@ -1893,10 +1918,10 @@ lbm_mbox_armored_stream(GMimeStream * stream)
     GMimeStream *fstream;
     GMimeFilter *filter;
     
-    fstream = g_mime_stream_filter_new_with_stream(stream);
+    fstream = g_mime_stream_filter_new(stream);
 
-    filter = g_mime_filter_crlf_new(GMIME_FILTER_CRLF_DECODE,
-				    GMIME_FILTER_CRLF_MODE_CRLF_ONLY);
+    filter = g_mime_filter_crlf_new(FALSE,
+				    FALSE);
     g_mime_stream_filter_add(GMIME_STREAM_FILTER(fstream), filter);
     g_object_unref(filter);
 
