@@ -20,9 +20,10 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 
-#define _XOPEN_SOURCE 500
 
 #include <stdio.h>
 #include <fcntl.h>
