@@ -24,10 +24,12 @@
    this file can be linked against the address book program balsa-ab
    without introducing extra dependencies. External library
    dependencies should go to libbalsa.c */
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "misc.h"
 
 #define _SVID_SOURCE           1
-#define _XOPEN_SOURCE          500
 #include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
@@ -39,13 +41,8 @@
 #include <sys/types.h>
 #include <sys/utsname.h>
 
-#ifdef HAVE_GNOME
-#include <libgnomevfs/gnome-vfs.h>
-#endif
-
 #include "libbalsa.h"
 #include "libbalsa_private.h"
-#include "misc.h"
 #include "html.h"
 #include <glib/gi18n.h>
 
@@ -490,16 +487,13 @@ LibBalsaCodeset
 libbalsa_set_fallback_codeset(LibBalsaCodeset codeset)
 {
     LibBalsaCodeset ret = sanitize_fallback_codeset;
-#if defined(HAVE_GMIME_2_2_7)
     const gchar *charsets[] = {
-        g_strdup("UTF-8"),
-        g_strdup(libbalsa_get_codeset_name(NULL, codeset)),
+        "UTF-8",
+        libbalsa_get_codeset_name(NULL, codeset),
         NULL
     };
 
     g_mime_set_user_charsets(charsets);
-    /* GMime will free the strings. */
-#endif                          /* HAVE_GMIME_2_2_7 */
 
     sanitize_fallback_codeset = codeset;
     return ret;
@@ -1040,22 +1034,6 @@ libbalsa_unlock_file (const char *path, int fd, int dot)
     return 0;
 }
 
-#if !GLIB_CHECK_VERSION(2, 2, 0)
-gboolean
-libbalsa_str_has_prefix(const gchar * str, const gchar * prefix)
-{
-    g_return_val_if_fail(str != NULL, FALSE);
-    g_return_val_if_fail(prefix != NULL, FALSE);
-
-    while (*prefix == *str && *prefix) {
-	++prefix;
-	++str;
-    }
-
-    return *prefix == '\0';
-}
-#endif				/* !GLIB_CHECK_VERSION(2, 2, 0) */
-
 
 /* libbalsa_ia_rfc2821_equal
    compares two addresses according to rfc2821: local-part@domain is equal,
@@ -1066,23 +1044,25 @@ libbalsa_ia_rfc2821_equal(const InternetAddress * a,
 			  const InternetAddress * b)
 {
     const gchar *a_atptr, *b_atptr;
+    const gchar *a_addr, *b_addr;
     gint a_atpos, b_atpos;
 
-    if (!a || !b || a->type != INTERNET_ADDRESS_NAME ||
-	b->type != INTERNET_ADDRESS_NAME)
+    if (!INTERNET_ADDRESS_IS_MAILBOX(a) || !INTERNET_ADDRESS_IS_MAILBOX(b))
         return FALSE;
 
     /* first find the "@" in the two addresses */
-    a_atptr = strchr(a->value.addr, '@');
-    b_atptr = strchr(b->value.addr, '@');
+    a_addr = INTERNET_ADDRESS_MAILBOX(a)->addr;
+    b_addr = INTERNET_ADDRESS_MAILBOX(b)->addr;
+    a_atptr = strchr(a_addr, '@');
+    b_atptr = strchr(b_addr, '@');
     if (!a_atptr || !b_atptr)
         return FALSE;
-    a_atpos = a_atptr - a->value.addr;
-    b_atpos = b_atptr - b->value.addr;
+    a_atpos = a_atptr - a_addr;
+    b_atpos = b_atptr - b_addr;
 
     /* now compare the strings */
     if (!a_atpos || !b_atpos || a_atpos != b_atpos || 
-        strncmp(a->value.addr, b->value.addr, a_atpos) ||
+        strncmp(a_addr, b_addr, a_atpos) ||
         g_ascii_strcasecmp(a_atptr, b_atptr))
         return FALSE;
     else
