@@ -23,6 +23,10 @@
 #ifndef __LIBBALSA_MISC_H__
 #define __LIBBALSA_MISC_H__
 
+#ifndef BALSA_VERSION
+# error "Include config.h before this file."
+#endif
+
 #include <stdio.h>
 #include <gtk/gtk.h>
 #include <gmime/gmime.h>
@@ -86,13 +90,12 @@ struct _LibBalsaCodesetInfo {
 };
 
 typedef void (*libbalsa_url_cb_t) (GtkTextBuffer *, GtkTextIter *,
-				   const gchar *, gpointer);
+				   const gchar *, guint, gpointer);
 typedef struct _LibBalsaUrlInsertInfo LibBalsaUrlInsertInfo;
 struct _LibBalsaUrlInsertInfo {
     libbalsa_url_cb_t callback;
     gpointer callback_data;
     gboolean buffer_is_flowed;
-    gchar *ml_url;
     GString *ml_url_buffer;
 };
 
@@ -123,7 +126,6 @@ gchar *libbalsa_wrap_rfc2646(gchar * par, gint width,
 void libbalsa_wrap_view(GtkTextView * view, gint length);
 void libbalsa_unwrap_buffer(GtkTextBuffer * buffer, GtkTextIter * iter,
                             gint lines);
-void libbalsa_prepare_delsp(GtkTextBuffer * buffer);
 
 const char* libbalsa_set_charset(const gchar * charset);
 const char* libbalsa_set_send_charset(const gchar * charset);
@@ -139,7 +141,7 @@ gboolean libbalsa_utf8_sanitize(gchar ** text, gboolean fallback,
 gboolean libbalsa_utf8_strstr(const gchar *s1,const gchar *s2);
 gboolean libbalsa_insert_with_url(GtkTextBuffer * buffer,
 				  const char *chars,
-				  const char *all_chars,
+				  guint len,
 				  GtkTextTag * tag,
 				  LibBalsaUrlInsertInfo *url_info);
 #if USE_GREGEX
@@ -156,11 +158,7 @@ int libbalsa_safe_open (const char *path, int flags, mode_t mode, GError **err);
 int libbalsa_lock_file (const char *path, int fd, int excl, int dot, int timeout);
 int libbalsa_unlock_file (const char *path, int fd, int dot);
 int libbalsa_safe_rename (const char *src, const char *target);
-#if GLIB_CHECK_VERSION(2, 2, 0)
 #define libbalsa_str_has_prefix(str, prefix) g_str_has_prefix((str), (prefix))
-#else				/* GLIB_CHECK_VERSION(2, 2, 0) */
-gboolean libbalsa_str_has_prefix(const gchar * str, const gchar * prefix);
-#endif				/* GLIB_CHECK_VERSION(2, 2, 0) */
 
 gboolean libbalsa_ia_rfc2821_equal(const InternetAddress * a,
 				   const InternetAddress * b);
