@@ -22,7 +22,9 @@
 #ifndef __RFC3156_H__
 #define __RFC3156_H__
 
-#include "config.h"
+#ifndef BALSA_VERSION
+# error "Include config.h before this file."
+#endif
 
 #ifdef HAVE_GPGME
 
