@@ -19,8 +19,9 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
-
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 #include "assistant_page_user.h"
 
 #include <sys/types.h>
@@ -86,7 +87,7 @@ balsa_druid_page_user_init(BalsaDruidPageUser * user,
 #endif
     label = GTK_LABEL(gtk_label_new(_(header2)));
     gtk_label_set_line_wrap(label, TRUE);
-    gtk_box_pack_start_defaults(GTK_BOX(page), GTK_WIDGET(label));
+    gtk_box_pack_start(GTK_BOX(page), GTK_WIDGET(label), TRUE, TRUE, 0);
 
     table = GTK_TABLE(gtk_table_new(10, 2, FALSE));
 
@@ -246,6 +247,7 @@ balsa_druid_page_user_next(GtkAssistant * druid, GtkWidget * page,
     const gchar *host, *mailbox;
     gchar *uhoh;
     LibBalsaIdentity *ident;
+    InternetAddress *ia;
 #if ENABLE_ESMTP
     LibBalsaSmtpServer *smtp_server;
 #endif /* ENABLE_ESMTP */
@@ -292,9 +294,9 @@ balsa_druid_page_user_next(GtkAssistant * druid, GtkWidget * page,
     } else {
         ident = balsa_app.current_ident;
     }
-    internet_address_set_name(ident->ia, mailbox);
-    internet_address_set_addr(ident->ia,
-                              gtk_entry_get_text(GTK_ENTRY(user->email)));
+    
+    ia = internet_address_mailbox_new (mailbox, gtk_entry_get_text(GTK_ENTRY(user->email)));
+    libbalsa_identity_set_address (ident, ia);
 
     /* outgoing mail */
 #if ENABLE_ESMTP
