@@ -19,9 +19,13 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 
+#include <stdlib.h>
 #include <string.h>
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #if HAVE_GNOME
 #include <gnome.h>
@@ -30,10 +34,15 @@
 # include <gconf/gconf.h>
 #endif
 
+
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
 
+#if HAVE_MACOSX_DESKTOP
+#include <ige-mac-integration.h>
+#endif
+
 #include "address-book.h"
 #include "address-book-vcard.h"
 #include "address-book-extern.h"
@@ -73,11 +82,13 @@ struct ABMainWindow {
 
 static void bab_cleanup(void);
 
+#if HAVE_GNOME
 static gint bab_save_session(GnomeClient * client, gint phase,
                              GnomeSaveStyle save_style, gint is_shutdown,
                              GnomeInteractStyle interact_style, gint is_fast,
                              gpointer client_data);
 static gint bab_kill_session(GnomeClient * client, gpointer client_data);
+#endif
 
 static void ab_set_edit_widget(LibBalsaAddress * address,
                                gboolean can_remove);
@@ -571,13 +582,7 @@ static GtkActionEntry entries[] = {
      N_("Add new entry"), G_CALLBACK(edit_new_entry_cb)},
     {"DeleteEntry", GTK_STOCK_NEW, N_("_Delete Entry"), NULL,
      N_("Delete entry"), G_CALLBACK(edit_delete_entry_cb)},
-    {"About",
-#if GTK_CHECK_VERSION(2, 6, 0)
-     GTK_STOCK_ABOUT,
-#else
-     GNOME_STOCK_ABOUT,
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
-     N_("_About"), NULL, NULL, NULL}
+    {"About", GTK_STOCK_ABOUT, N_("_About"), NULL, NULL, NULL}
 };
 
 static const char *ui_description =
@@ -621,8 +626,11 @@ get_main_menu(GtkWidget * window, GtkWidget ** menubar,
     GtkActionGroup *action_group;
     GtkUIManager *ui_manager;
     GtkAccelGroup *accel_group;
-    GError *error;
+    GError *error = NULL;
     GList *ab;
+#if HAVE_MACOSX_DESKTOP
+    IgeMacMenuGroup *group;
+#endif
 
     contacts_app.action_group = action_group =
         gtk_action_group_new("MenuActions");
@@ -636,7 +644,6 @@ get_main_menu(GtkWidget * window, GtkWidget ** menubar,
     accel_group = gtk_ui_manager_get_accel_group(ui_manager);
     gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
 
-    error = NULL;
     if (!gtk_ui_manager_add_ui_from_string(ui_manager, ui_description,
                                            -1, &error)) {
         g_message("building menus failed: %s", error->message);
@@ -646,6 +653,16 @@ get_main_menu(GtkWidget * window, GtkWidget ** menubar,
 
     for (ab = address_books; ab; ab = ab->next)
         add_address_book(LIBBALSA_ADDRESS_BOOK(ab->data));
+    
+#if HAVE_MACOSX_DESKTOP
+    ige_mac_menu_set_menu_bar(GTK_MENU_SHELL(gtk_ui_manager_get_widget(ui_manager, "/MainMenu")));
+    ige_mac_menu_set_quit_menu_item(GTK_MENU_ITEM(gtk_ui_manager_get_widget(ui_manager, "/MainMenu/FileMenu/Quit")));
+
+    group = ige_mac_menu_add_app_menu_group();
+    ige_mac_menu_add_app_menu_item(group,
+				   GTK_MENU_ITEM(gtk_ui_manager_get_widget(ui_manager, "/MainMenu/HelpMenu/About")), 
+                                   NULL);
+#endif
 
     if (menubar)
         /* Finally, return the actual menu bar created by the UIManager. */
@@ -986,9 +1003,11 @@ bab_window_new()
     gtk_container_add(GTK_CONTAINER(wnd), main_vbox);
 
     get_main_menu(GTK_WIDGET(wnd), &menubar, contacts_app.address_book_list);
+#ifndef HAVE_MACOSX_DESKTOP
     if (menubar)
         gtk_box_pack_start(GTK_BOX(main_vbox),
                            menubar, FALSE, FALSE, 1);
+#endif
 
     contacts_app.notebook = gtk_notebook_new();
     gtk_box_pack_start(GTK_BOX(main_vbox),
@@ -1089,9 +1108,8 @@ information_real(void)
 int
 main(int argc, char *argv[])
 {
+#if HAVE_GNOME
     GnomeClient *client;
-#ifdef GTKHTML_HAVE_GCONF
-    GError *gconf_error;
 #endif
 
 #ifdef ENABLE_NLS
@@ -1104,6 +1122,7 @@ main(int argc, char *argv[])
 
     /* FIXME: do we need to allow a non-GUI mode? */
     gtk_init_check(&argc, &argv);
+#if HAVE_GNOME
     gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
 #ifndef GNOME_PARAM_GOPTION_CONTEXT
                        GNOME_PARAM_POPT_TABLE, NULL,
@@ -1114,11 +1133,10 @@ main(int argc, char *argv[])
                        GNOME_PARAM_GOPTION_CONTEXT, NULL,
                        GNOME_PARAM_NONE);
 #endif
+#endif
 
 #ifdef GTKHTML_HAVE_GCONF
-    if (!gconf_init(argc, argv, &gconf_error))
-	g_error_free(gconf_error);
-    gconf_error = NULL;
+    gconf_init(argc, argv, NULL);
 #endif
 
     bab_init();
@@ -1135,7 +1153,7 @@ main(int argc, char *argv[])
     LIBBALSA_TYPE_ADDRESS_BOOK_RUBRICA;
 #endif /* HAVE_RUBRICA */
     libbalsa_real_information_func = (LibBalsaInformationFunc)information_real;
-    g_mime_init(0);
+    g_mime_init(GMIME_ENABLE_RFC2047_WORKAROUNDS);
 
     /* load address book data */
     libbalsa_conf_push_group("Globals");
@@ -1153,11 +1171,13 @@ main(int argc, char *argv[])
                      G_CALLBACK(bab_delete_ok), NULL);
 
     /* session management */
+#if HAVE_GNOME
     client = gnome_master_client();
     g_signal_connect(G_OBJECT(client), "save_yourself",
 		     G_CALLBACK(bab_save_session), argv[0]);
     g_signal_connect(G_OBJECT(client), "die",
 		     G_CALLBACK(bab_kill_session), NULL);
+#endif
 
     gtk_widget_show_all(ab_window);
     gtk_widget_hide(contacts_app.edit_widget);
@@ -1186,6 +1206,7 @@ bab_cleanup(void)
     gtk_main_quit();
 }
 
+#if HAVE_GNOME
 static gint
 bab_kill_session(GnomeClient * client, gpointer client_data)
 {
@@ -1215,3 +1236,4 @@ bab_save_session(GnomeClient * client, gint phase,
 
     return TRUE;
 }
+#endif /* HAVE_GNOME */
