@@ -19,7 +19,10 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "ab-window.h"
 
 #include <string.h>
 #include <glib/gi18n.h>
@@ -27,7 +30,6 @@
 #include "balsa-app.h"
 #include "sendmsg-window.h"
 #include "save-restore.h"
-#include "ab-window.h"
 
 enum {
     LIST_COLUMN_NAME,
