@@ -19,11 +19,14 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "balsa-app.h"
+
 #include <string.h>
+#include <stdlib.h>
 #ifdef BALSA_USE_THREADS
-/* _XOPEN_SOURCE is needed for rwlocks */
-#define _XOPEN_SOURCE 500
 #include <pthread.h>
 #endif
 
@@ -36,7 +39,6 @@
 #include "misc.h"
 #include "server.h"
 #include "smtp-server.h"
-#include "balsa-app.h"
 #include "save-restore.h"
 
 #include <glib/gi18n.h>	/* Must come after balsa-app.h. */
@@ -57,6 +59,8 @@ const gchar *pspell_suggest_modes[] = {
 };
 #endif                          /* HAVE_GTKSPELL */
 
+#define HIG_PADDING 12
+
 /* ask_password:
    asks the user for the password to the mailbox on given remote server.
 */
@@ -65,6 +69,13 @@ ask_password_real(LibBalsaServer * server, LibBalsaMailbox * mbox)
 {
     GtkWidget *dialog, *entry, *rememb;
     gchar *prompt, *passwd = NULL;
+#if defined(HAVE_GNOME_KEYRING)
+    static const gchar *remember_password_message =
+        N_("_Remember password in keyring");
+#else
+    static const gchar *remember_password_message =
+        N_("_Remember password");
+#endif
 
     g_return_val_if_fail(server != NULL, NULL);
     if (mbox)
@@ -74,8 +85,8 @@ ask_password_real(LibBalsaServer * server, LibBalsaMailbox * mbox)
 			    mbox->name, server->user, server->host);
     else
 	prompt =
-	    g_strdup_printf(_("Mailbox _password for %s@%s:"), server->user,
-			    server->host);
+	    g_strdup_printf(_("_Password for %s@%s (%s):"), server->user,
+			    server->host, server->protocol);
 
     dialog = gtk_dialog_new_with_buttons(_("Password needed"),
                                          GTK_WINDOW(balsa_app.main_window),
@@ -83,6 +94,7 @@ ask_password_real(LibBalsaServer * server, LibBalsaMailbox * mbox)
                                          GTK_STOCK_OK, GTK_RESPONSE_OK,
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                          NULL); 
+    gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox), HIG_PADDING);
     gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),
                       gtk_label_new_with_mnemonic(prompt));
     g_free(prompt);
@@ -91,7 +103,7 @@ ask_password_real(LibBalsaServer * server, LibBalsaMailbox * mbox)
     gtk_entry_set_width_chars(GTK_ENTRY(entry), 20);
     gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
 
-    rememb =  gtk_check_button_new_with_mnemonic(_("_Remember password"));
+    rememb =  gtk_check_button_new_with_mnemonic(_(remember_password_message));
     gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), rememb);
     if(server->remember_passwd)
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rememb), TRUE);
@@ -316,7 +328,7 @@ balsa_app_init(void)
     balsa_app.expand_tree = FALSE;
     balsa_app.show_mblist = TRUE;
     balsa_app.show_notebook_tabs = FALSE;
-    balsa_app.alternative_layout = FALSE;
+    balsa_app.layout_type = LAYOUT_DEFAULT;
     balsa_app.view_message_on_open = TRUE;
     balsa_app.mw_action_after_move = NEXT_UNREAD;
 
@@ -340,10 +352,13 @@ balsa_app_init(void)
     balsa_app.mblist_totalmsg_width = TOTALMSGCOUNT_DEFAULT_WIDTH;
 
     /* Allocate the best colormap we can get */
-    gtk_widget_set_default_colormap(gdk_rgb_get_colormap());
     balsa_app.visual = gdk_visual_get_best();
-    balsa_app.colormap = gdk_colormap_new(balsa_app.visual, TRUE);
-
+    if (!(balsa_app.colormap = gdk_colormap_new(balsa_app.visual, TRUE))) {
+	balsa_app.visual = gdk_visual_get_system();
+	balsa_app.colormap = gdk_colormap_get_system();
+    }
+    g_assert(balsa_app.colormap);
+    
     /* arp */
     balsa_app.quote_str = NULL;
 
@@ -366,20 +381,14 @@ balsa_app_init(void)
     balsa_app.date_string = g_strdup(DEFAULT_DATE_FORMAT);
 
     /* printing */
-#ifdef HAVE_GTK_PRINT
     balsa_app.print_settings = gtk_print_settings_new();
     balsa_app.page_setup = gtk_page_setup_new();
-#else
-    balsa_app.paper_size = g_strdup(DEFAULT_PAPER_SIZE);
-#endif
 
     balsa_app.print_header_font = g_strdup(DEFAULT_PRINT_HEADER_FONT);
     balsa_app.print_footer_font = g_strdup(DEFAULT_PRINT_FOOTER_FONT);
     balsa_app.print_body_font   = g_strdup(DEFAULT_PRINT_BODY_FONT);
     balsa_app.print_highlight_cited = FALSE;
-#ifdef HAVE_GTK_PRINT
     balsa_app.print_highlight_phrases = FALSE;
-#endif
 
     /* address book */
     balsa_app.address_book_list = NULL;
@@ -410,14 +419,7 @@ balsa_app_init(void)
 
     balsa_app.notify_new_mail_sound = 1;
     balsa_app.notify_new_mail_dialog = 0;
-#if GTK_CHECK_VERSION(2, 10, 0)
     balsa_app.notify_new_mail_icon = 1;
-#endif                          /* GTK_CHECK_VERSION(2, 10, 0) */
-
-#if !GTK_CHECK_VERSION(2, 11, 0)
-    /* Tooltips */
-    balsa_app.tooltips = gtk_tooltips_new();
-#endif                          /* !GTK_CHECK_VERSION(2, 11, 0) */
 
     /* Local and IMAP */
     balsa_app.local_scan_depth = 1;
@@ -440,18 +442,12 @@ void
 balsa_app_destroy(void)
 {
     config_views_save();
-    if (balsa_app.empty_trash_on_exit)
-	empty_trash(balsa_app.main_window);
-
     config_save();
 
     g_list_foreach(balsa_app.address_book_list, (GFunc)g_object_unref, NULL);
     g_list_free(balsa_app.address_book_list);
     balsa_app.address_book_list = NULL;
 
-    /* close all mailboxes */
-    gtk_widget_destroy(balsa_app.notebook);
-
     /* now free filters */
     g_slist_foreach(balsa_app.filters, (GFunc)libbalsa_filter_free, 
 		    GINT_TO_POINTER(TRUE));
@@ -505,8 +501,8 @@ update_timer(gboolean update, guint minutes)
  */
 
 static void
-open_mailboxes_idle_func(const gchar * url, LibBalsaMailboxView * view,
-                         GString * str)
+append_url_if_open(const gchar * url, LibBalsaMailboxView * view,
+                   GString * str)
 {
     if (view->open) {
         if (str->len)
@@ -533,7 +529,7 @@ open_mailboxes_idle_cb(gchar ** urls)
 
         str = g_string_new(NULL);
         g_hash_table_foreach(libbalsa_mailbox_view_table,
-                             (GHFunc) open_mailboxes_idle_func, str);
+                             (GHFunc) append_url_if_open, str);
         urls = g_strsplit(str->str, ";", 0);
         g_string_free(str, TRUE);
     }
