@@ -20,11 +20,12 @@
  */
 
 #include <string.h>
-#include <gdk/gdkx.h>
-
-#include "config.h"
 
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 #include "balsa-icons.h"
+
 #include "libbalsa.h"
 #include "mailbox.h"
 #include "address-view.h"
@@ -64,9 +65,7 @@ balsa_icon_create(const gchar ** data, GdkPixmap ** pmap, GdkBitmap ** bmap)
                                          bmap, 0, (gchar **) data);
 }
 
-#if GTK_CHECK_VERSION(2, 8, 0)
 static GHashTable *balsa_icon_table;
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
 
 static void
 load_balsa_pixmap(GtkIconTheme *icon_theme, GtkIconFactory *factory, 
@@ -109,10 +108,8 @@ load_balsa_pixmap(GtkIconTheme *icon_theme, GtkIconFactory *factory,
     } else
 	use_id = bpixmap->stock_id;
 
-#if GTK_CHECK_VERSION(2, 8, 0)
     g_hash_table_insert(balsa_icon_table, g_strdup(bpixmap->name), 
                         g_strdup(use_id));
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
 
     if (!gtk_icon_size_lookup(bpixmap->sizes[0], &width, &height)) {
 	BICONS_ERR("failed: could not look up default icon size %d",
@@ -142,8 +139,7 @@ load_balsa_pixmap(GtkIconTheme *icon_theme, GtkIconFactory *factory,
 	    if (!pixbuf) {
 		BICONS_ERR("additional size %d failed: %s", width,
 			   error->message);
-		g_error_free(error);
-		error = NULL;
+		g_clear_error(&error);
 	    } else {
 		GtkIconSource *icon_source = gtk_icon_source_new();
 		
@@ -160,6 +156,7 @@ load_balsa_pixmap(GtkIconTheme *icon_theme, GtkIconFactory *factory,
     }
 
     gtk_icon_factory_add(factory, bpixmap->name, icon_set);
+    gtk_icon_set_unref(icon_set);
 }
 
 void
@@ -294,10 +291,8 @@ register_balsa_pixmaps(void)
     GtkIconFactory *factory = gtk_icon_factory_new();
     GtkIconTheme *icon_theme = gtk_icon_theme_get_default();
 
-#if GTK_CHECK_VERSION(2, 8, 0)
     balsa_icon_table =
         g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
     gtk_icon_factory_add_default(factory);
     gtk_icon_theme_append_search_path(icon_theme, BALSA_DATA_PREFIX);
 
@@ -340,7 +335,6 @@ register_balsa_pixbufs(GtkWidget * widget)
 						 NULL));
 }
 
-#if GTK_CHECK_VERSION(2, 8, 0)
 const gchar *
 balsa_icon_id(const gchar * name)
 {
@@ -348,4 +342,3 @@ balsa_icon_id(const gchar * name)
 
     return retval ? retval : name;
 }
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
