@@ -22,7 +22,11 @@
 #ifndef __BALSA_ICONS_H__
 #define __BALSA_ICONS_H__
 
-#include "config.h"
+#ifndef BALSA_VERSION
+# error "Include config.h before this file."
+#endif
+
+#include <gtk/gtk.h>
 
 #define BALSA_PIXMAP_ATTACHMENT			"balsa_attachment"
 #define BALSA_PIXMAP_COMPOSE			"balsa_compose"
@@ -37,6 +41,7 @@
 #define BALSA_PIXMAP_PREVIOUS			"balsa_previous"
 #define BALSA_PIXMAP_PREVIOUS_PART		"balsa_previous_part"
 #define BALSA_PIXMAP_POSTPONE			"balsa_postpone"
+#define BALSA_PIXMAP_REQUEST_MDN                "balsa_request_mdn"
 #define BALSA_PIXMAP_SEND			"balsa_send"
 #define BALSA_PIXMAP_SEND_RECEIVE		"balsa_send_receive"
 #define BALSA_PIXMAP_TRASH_EMPTY		"balsa_trash_empty"
@@ -92,7 +97,5 @@ void register_balsa_pixmaps(void);
 void register_balsa_pixbufs(GtkWidget * widget);
 void balsa_icon_create(const gchar ** data, GdkPixmap ** pmap,
                        GdkBitmap ** bmap);
-#if GTK_CHECK_VERSION(2, 8, 0)
 const gchar * balsa_icon_id(const gchar * name);
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
 #endif
