@@ -30,7 +30,10 @@
    key and replace the key on every change of the sort method.  
 */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "balsa-index.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -38,7 +41,6 @@
 
 #include "balsa-app.h"
 #include "balsa-icons.h"
-#include "balsa-index.h"
 #include "balsa-mblist.h"
 #include "balsa-message.h"
 #include "main-window.h"
@@ -51,13 +53,8 @@
 #include <glib/gi18n.h>
 
 /* TREE_VIEW_FIXED_HEIGHT enables hight-performance mode of GtkTreeView
- * very useful for large mailboxes (#msg >5000) but: a. is available only
- * in gtk2>=2.3.5 b. may expose some bugs in gtk.
- * gtk-2.4.9 has been tested with a positive result.
- */
-#if GTK_CHECK_VERSION(2,4,9)
+ * very useful for large mailboxes (#msg >5000)  */
 #define TREE_VIEW_FIXED_HEIGHT 1
-#endif
 
 
 /* gtk widget */
@@ -92,6 +89,10 @@ static void bndx_column_resize(GtkWidget * widget,
 static void bndx_tree_expand_cb(GtkTreeView * tree_view,
                                 GtkTreeIter * iter, GtkTreePath * path,
                                 gpointer user_data);
+static gboolean bndx_test_collapse_row_cb(GtkTreeView * tree_view,
+                                          GtkTreeIter * iter,
+                                          GtkTreePath * path,
+                                          gpointer user_data);
 static void bndx_tree_collapse_cb(GtkTreeView * tree_view,
                                   GtkTreeIter * iter, GtkTreePath * path,
                                   gpointer user_data);
@@ -290,10 +291,14 @@ bndx_instance_init(BalsaIndex * index)
     
     /* Index column */
     renderer = gtk_cell_renderer_text_new();
-    column =
-        gtk_tree_view_column_new_with_attributes("#", renderer,
-                                                 "text", LB_MBOX_MSGNO_COL,
-                                                 NULL);
+    column = gtk_tree_view_column_new_with_attributes
+        ("#", renderer,
+         "text",           LB_MBOX_MSGNO_COL,
+         "foreground",     LB_MBOX_FOREGROUND_COL,
+         "foreground-set", LB_MBOX_FOREGROUND_SET_COL,
+         "background",     LB_MBOX_BACKGROUND_COL,
+         "background-set", LB_MBOX_BACKGROUND_SET_COL,
+         NULL);
     g_object_set(renderer, "xalign", 1.0, NULL);
     set_sizing(column); gtk_tree_view_append_column(tree_view, column);
     bi_apply_other_column_settings(column, TRUE, LB_MBOX_MSGNO_COL);
@@ -301,31 +306,39 @@ bndx_instance_init(BalsaIndex * index)
     /* Status icon column */
     renderer = gtk_cell_renderer_pixbuf_new();
     gtk_cell_renderer_set_fixed_size(renderer, icon_w, icon_h);
-    column =
-        gtk_tree_view_column_new_with_attributes("S", renderer,
-                                                 "pixbuf", LB_MBOX_MARKED_COL,
-                                                 NULL);
+    column = gtk_tree_view_column_new_with_attributes
+        ("S", renderer,
+         "pixbuf",              LB_MBOX_MARKED_COL,
+         "cell-background",     LB_MBOX_BACKGROUND_COL,
+         "cell-background-set", LB_MBOX_BACKGROUND_SET_COL,
+         NULL);
     set_sizing(column); gtk_tree_view_append_column(tree_view, column);
     bi_apply_other_column_settings(column, FALSE, 0);
 
     /* Attachment icon column */
     renderer = gtk_cell_renderer_pixbuf_new();
     gtk_cell_renderer_set_fixed_size(renderer, icon_w, icon_h);
-    column =
-        gtk_tree_view_column_new_with_attributes("A", renderer,
-                                                 "pixbuf", LB_MBOX_ATTACH_COL,
-                                                 NULL);
+    column = gtk_tree_view_column_new_with_attributes
+        ("A", renderer,
+         "pixbuf",              LB_MBOX_ATTACH_COL,
+         "cell-background",     LB_MBOX_BACKGROUND_COL,
+         "cell-background-set", LB_MBOX_BACKGROUND_SET_COL,
+         NULL);
     set_sizing(column); gtk_tree_view_append_column(tree_view, column);
     bi_apply_other_column_settings(column, FALSE, 0);
 
     /* From/To column */
     renderer = gtk_cell_renderer_text_new();
-    column = 
-        gtk_tree_view_column_new_with_attributes(_("From"), renderer,
-                                                 "text", LB_MBOX_FROM_COL,
-						 "weight", LB_MBOX_WEIGHT_COL,
-						 "style", LB_MBOX_STYLE_COL,
-                                                 NULL);
+    column = gtk_tree_view_column_new_with_attributes
+        (_("From"), renderer,
+         "text",           LB_MBOX_FROM_COL,
+         "weight",         LB_MBOX_WEIGHT_COL,
+         "style",          LB_MBOX_STYLE_COL,
+         "foreground",     LB_MBOX_FOREGROUND_COL,
+         "foreground-set", LB_MBOX_FOREGROUND_SET_COL,
+         "background",     LB_MBOX_BACKGROUND_COL,
+         "background-set", LB_MBOX_BACKGROUND_SET_COL,
+         NULL);
     gtk_tree_view_column_set_resizable(column, TRUE);
     gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
     gtk_tree_view_append_column(tree_view, column);
@@ -333,12 +346,16 @@ bndx_instance_init(BalsaIndex * index)
 
     /* Subject column--contains tree expanders */
     renderer = gtk_cell_renderer_text_new();
-    column = 
-        gtk_tree_view_column_new_with_attributes(_("Subject"), renderer,
-                                                 "text", LB_MBOX_SUBJECT_COL,
-						 "weight", LB_MBOX_WEIGHT_COL,
-						 "style", LB_MBOX_STYLE_COL,
-                                                 NULL);
+    column = gtk_tree_view_column_new_with_attributes
+        (_("Subject"), renderer,
+         "text",           LB_MBOX_SUBJECT_COL,
+         "weight",         LB_MBOX_WEIGHT_COL,
+         "style",          LB_MBOX_STYLE_COL,
+         "foreground",     LB_MBOX_FOREGROUND_COL,
+         "foreground-set", LB_MBOX_FOREGROUND_SET_COL,
+         "background",     LB_MBOX_BACKGROUND_COL,
+         "background-set", LB_MBOX_BACKGROUND_SET_COL,
+         NULL);
     gtk_tree_view_column_set_resizable(column, TRUE);
     gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
     gtk_tree_view_append_column(tree_view, column);
@@ -347,12 +364,16 @@ bndx_instance_init(BalsaIndex * index)
 
     /* Date column */
     renderer = gtk_cell_renderer_text_new();
-    column = 
-        gtk_tree_view_column_new_with_attributes(_("Date"), renderer,
-                                                 "text", LB_MBOX_DATE_COL,
-						 "weight", LB_MBOX_WEIGHT_COL,
-						 "style", LB_MBOX_STYLE_COL,
-                                                 NULL);
+    column = gtk_tree_view_column_new_with_attributes
+        (_("Date"), renderer,
+         "text",           LB_MBOX_DATE_COL,
+         "weight",         LB_MBOX_WEIGHT_COL,
+         "style",          LB_MBOX_STYLE_COL,
+         "foreground",     LB_MBOX_FOREGROUND_COL,
+         "foreground-set", LB_MBOX_FOREGROUND_SET_COL,
+         "background",     LB_MBOX_BACKGROUND_COL,
+         "background-set", LB_MBOX_BACKGROUND_SET_COL,
+         NULL);
     gtk_tree_view_column_set_resizable(column, TRUE);
     gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
     gtk_tree_view_append_column(tree_view, column);
@@ -364,23 +385,23 @@ bndx_instance_init(BalsaIndex * index)
     renderer = gtk_cell_renderer_text_new();
     g_object_set(renderer, "xalign", 1.0, NULL);
     gtk_tree_view_column_pack_start(column, renderer, FALSE);
-    gtk_tree_view_column_set_attributes(column, renderer,
-                                        "text", LB_MBOX_SIZE_COL,
-					"weight", LB_MBOX_WEIGHT_COL,
-					"style", LB_MBOX_STYLE_COL,
-                                        NULL);
+    gtk_tree_view_column_set_attributes
+        (column, renderer,
+         "text",           LB_MBOX_SIZE_COL,
+         "weight",         LB_MBOX_WEIGHT_COL,
+         "style",          LB_MBOX_STYLE_COL,
+         "foreground",     LB_MBOX_FOREGROUND_COL,
+         "foreground-set", LB_MBOX_FOREGROUND_SET_COL,
+         "background",     LB_MBOX_BACKGROUND_COL,
+         "background-set", LB_MBOX_BACKGROUND_SET_COL,
+         NULL);
     set_sizing(column); gtk_tree_view_append_column(tree_view, column);
     bi_apply_other_column_settings(column, TRUE, LB_MBOX_SIZE_COL);
 
     /* Initialize some other members */
     index->mailbox_node = NULL;
     index->popup_menu = bndx_popup_menu_create(index);
-#if GLIB_CHECK_VERSION(2, 10, 0)
     g_object_ref_sink(index->popup_menu);
-#else                           /* GLIB_CHECK_VERSION(2, 10, 0) */
-    g_object_ref(index->popup_menu);
-    gtk_object_sink(GTK_OBJECT(index->popup_menu));
-#endif                          /* GLIB_CHECK_VERSION(2, 10, 0) */
     
     gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
 
@@ -401,6 +422,8 @@ bndx_instance_init(BalsaIndex * index)
     index->row_expanded_id =
         g_signal_connect_after(tree_view, "row-expanded",
                                G_CALLBACK(bndx_tree_expand_cb), NULL);
+    g_signal_connect(tree_view, "test-collapse-row",
+                     G_CALLBACK(bndx_test_collapse_row_cb), NULL);
     index->row_collapsed_id =
         g_signal_connect_after(tree_view, "row-collapsed",
                                G_CALLBACK(bndx_tree_collapse_cb), NULL);
@@ -409,9 +432,7 @@ bndx_instance_init(BalsaIndex * index)
     g_signal_connect_after(tree_view, "size-allocate",
                            G_CALLBACK(bndx_column_resize),
                            NULL);
-#if GTK_CHECK_VERSION(2,4,9)
     gtk_tree_view_set_enable_search(tree_view, FALSE);
-#endif
 
     gtk_drag_source_set(GTK_WIDGET (index), 
                         GDK_BUTTON1_MASK | GDK_SHIFT_MASK | GDK_CONTROL_MASK,
@@ -524,17 +545,31 @@ bndx_selection_changed(GtkTreeSelection * selection, BalsaIndex * index)
 
     if (index->current_msgno) {
         GtkTreePath *path;
+
         if (libbalsa_mailbox_msgno_find(index->mailbox_node->mailbox,
                                         index->current_msgno,
                                         &path, NULL)) {
-            gboolean ok =
-                gtk_tree_selection_path_is_selected(selection, path)
-                || !bndx_row_is_viewable(index, path);
+            gboolean update_preview = TRUE;
+
+            if (gtk_tree_selection_path_is_selected(selection, path)) {
+                /* The current message is still selected; we leave the
+                 * preview unchanged. */
+                update_preview = FALSE;
+            } else if (index->collapsing
+                       && !bndx_row_is_viewable(index, path)) {
+                /* The current message was deselected because its thread
+                 * was collapsed; we leave the preview unchanged, and to
+                 * avoid confusion, we unselect all messages. */
+                g_signal_handler_block(selection,
+                                       index->selection_changed_id);
+                gtk_tree_selection_unselect_all(selection);
+                g_signal_handler_unblock(selection,
+                                         index->selection_changed_id);
+                update_preview = FALSE;
+            }
             gtk_tree_path_free(path);
-            if (ok)
-                /* Either the current message is still selected, or it
-                 * was hidden by collapsing its thread; in either case,
-                 * we leave the preview unchanged. */
+
+            if (!update_preview)
                 return;
         }
     }
@@ -696,7 +731,21 @@ bndx_tree_expand_cb(GtkTreeView * tree_view, GtkTreeIter * iter,
     bndx_changed_find_row(index);
 }
 
+/* bndx_test_collapse_row_cb:
+ * callback on "test-collapse-row" events
+ * just set collapsing, and return FALSE to allow the row (thread) to be
+ * collapsed. */
+static gboolean
+bndx_test_collapse_row_cb(GtkTreeView * tree_view, GtkTreeIter * iter,
+                          GtkTreePath * path, gpointer user_data)
+{
+    BalsaIndex *index = BALSA_INDEX(tree_view);
+    index->collapsing = TRUE;
+    return FALSE;
+}
+
 /* callback on collapse events;
+ * clear collapsing;
  * the next message may have become invisible, so we must check whether
  * a next message still exists. */
 static void
@@ -704,6 +753,7 @@ bndx_tree_collapse_cb(GtkTreeView * tree_view, GtkTreeIter * iter,
                       GtkTreePath * path, gpointer user_data)
 {
     BalsaIndex *index = BALSA_INDEX(tree_view);
+    index->collapsing = FALSE;
     bndx_changed_find_row(index);
 }
 
@@ -1004,6 +1054,31 @@ balsa_index_load_mailbox_node (BalsaIndex * index,
     return FALSE;
 }
 
+void
+balsa_index_set_width_preference(BalsaIndex *bindex,
+                                 BalsaIndexWidthPreference pref)
+{
+    GtkTreeView *tree_view;
+    gboolean visible;
+
+    if (pref == bindex->width_preference)
+        return;
+
+    bindex->width_preference = pref;
+    switch (pref) {
+    case BALSA_INDEX_NARROW: visible = FALSE; break;
+    default:
+    case BALSA_INDEX_WIDE:   visible = TRUE;  break;
+    }
+
+    tree_view = GTK_TREE_VIEW(bindex);
+    gtk_tree_view_column_set_visible
+        (gtk_tree_view_get_column(tree_view, LB_MBOX_MSGNO_COL), visible);
+    gtk_tree_view_column_set_visible
+        (gtk_tree_view_get_column(tree_view, LB_MBOX_ATTACH_COL), visible);
+    gtk_tree_view_column_set_visible
+        (gtk_tree_view_get_column(tree_view, LB_MBOX_SIZE_COL), visible);
+}
 
 /*
  * select message interfaces
@@ -2409,7 +2484,7 @@ bndx_pipe_queue_last(struct BndxPipeQueue *queue)
     int std_input;
     int std_output;
     int std_error;
-    guint msgno;
+    guint msgno = 0;
 
     while(queue->msgnos->len>0){
 	msgno = g_array_index(queue->msgnos, guint, queue->msgnos->len-1);
@@ -2550,14 +2625,8 @@ bndx_pipe_response(GtkWidget * dialog, gint response,
         gchar *pipe_cmd;
         GList *active_cmd;
 
-#if GTK_CHECK_VERSION(2, 6, 0)
         pipe_cmd =
             gtk_combo_box_get_active_text(GTK_COMBO_BOX(info->entry));
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-        pipe_cmd =
-            gtk_editable_get_chars(GTK_EDITABLE
-                                   (GTK_BIN(info->entry)->child), 0, -1);
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
         active_cmd =
             g_list_find_custom(balsa_app.pipe_cmds, pipe_cmd,
                                (GCompareFunc) strcmp);
@@ -2663,14 +2732,9 @@ balsa_index_ensure_visible(BalsaIndex * index)
         /* Current message not displayed, make sure that something
            else is... */
         gtk_tree_view_get_visible_rect(tree_view, &rect);
-#if GTK_CHECK_VERSION(2, 11, 0)
         gtk_tree_view_convert_tree_to_widget_coords(tree_view,
                                                     rect.x, rect.y,
                                                     &rect.x, &rect.y);
-#else                           /* GTK_CHECK_VERSION(2, 11, 0) */
-        gtk_tree_view_tree_to_widget_coords(tree_view, rect.x, rect.y,
-                                            &rect.x, &rect.y);
-#endif                          /* GTK_CHECK_VERSION(2, 11, 0) */
 
         if (gtk_tree_view_get_path_at_pos(tree_view, rect.x, rect.y, &path,
                                           NULL, NULL, NULL)) {
@@ -2722,3 +2786,49 @@ balsa_index_count_selected_messages(BalsaIndex * bindex)
         gtk_tree_selection_count_selected_rows(gtk_tree_view_get_selection
                                                (GTK_TREE_VIEW(bindex)));
 }
+
+/* Select all messages in current thread */
+void
+balsa_index_select_thread(BalsaIndex * bindex)
+{
+    GtkTreeIter iter;
+    GtkTreeModel *model = GTK_TREE_MODEL(bindex->mailbox_node->mailbox);
+    GtkTreeIter next_iter;
+    GtkTreePath *path;
+    GtkTreeSelection *selection =
+        gtk_tree_view_get_selection(GTK_TREE_VIEW(bindex));
+    gboolean valid;
+
+    if (!bindex->current_msgno
+        || !libbalsa_mailbox_msgno_find(bindex->mailbox_node->mailbox,
+                                        bindex->current_msgno, NULL,
+                                        &iter))
+        return;
+
+    while (gtk_tree_model_iter_parent(model, &next_iter, &iter))
+        iter = next_iter;
+
+    path = gtk_tree_model_get_path(model, &iter);
+    gtk_tree_view_expand_row(GTK_TREE_VIEW(bindex), path, TRUE);
+    gtk_tree_path_free(path);
+
+    do {
+        gtk_tree_selection_select_iter(selection, &iter);
+
+        valid = gtk_tree_model_iter_children(model, &next_iter, &iter);
+        if (valid)
+            iter = next_iter;
+        else {
+            do {
+                GtkTreeIter save_iter = iter;
+
+                valid = gtk_tree_model_iter_next(model, &iter);
+                if (valid)
+                    break;
+                valid = gtk_tree_model_iter_parent(model, &iter,
+                                                   &save_iter);
+            } while (valid);
+        }
+    } while (valid
+             && gtk_tree_model_iter_parent(model, &next_iter, &iter));
+}
