@@ -19,16 +19,17 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "balsa-mblist.h"
 
 /* #include <gtk/gtkfeatures.h> */
 #include <string.h>
-#include <gdk/gdkfont.h>
 
 #include "balsa-app.h"
 #include "balsa-icons.h"
 #include "balsa-index.h"
-#include "balsa-mblist.h"
 #include "libbalsa.h"
 #include <glib/gi18n.h>
 #include "mailbox-node.h"
@@ -224,11 +225,7 @@ bmbl_set_property_node_style(GSList * list)
             GtkTreeModel *model;
             GtkTreeIter iter;
 
-#if GTK_CHECK_VERSION(2, 8, 0)
             model = gtk_tree_row_reference_get_model(reference);
-#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
-            model = GTK_TREE_MODEL(balsa_app.mblist_tree_store);
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
             if (gtk_tree_model_get_iter(model, &iter, path))
                 bmbl_node_style(model, &iter);
             gtk_tree_path_free(path);
@@ -358,11 +355,7 @@ bmbl_init(BalsaMBList * mblist)
     renderer = gtk_cell_renderer_pixbuf_new();
     gtk_tree_view_column_pack_start(column, renderer, FALSE);
     gtk_tree_view_column_set_attributes(column, renderer,
-#if GTK_CHECK_VERSION(2, 8, 0)
                                         "icon-name", ICON_COLUMN,
-#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
-                                        "pixbuf", ICON_COLUMN,
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
                                         NULL);
     renderer = gtk_cell_renderer_text_new();
     gtk_tree_view_column_pack_start(column, renderer, FALSE);
@@ -486,11 +479,7 @@ balsa_mblist_get_store(void)
         balsa_app.mblist_tree_store =
             gtk_tree_store_new(N_COLUMNS,
                                G_TYPE_OBJECT,     /* MBNODE_COLUMN */
-#if GTK_CHECK_VERSION(2, 8, 0)
                                G_TYPE_STRING,     /* ICON_COLUMN   */
-#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
-                               GDK_TYPE_PIXBUF,   /* ICON_COLUMN   */
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
                                G_TYPE_STRING,     /* NAME_COLUMN   */
                                PANGO_TYPE_WEIGHT, /* WEIGHT_COLUMN */
                                PANGO_TYPE_STYLE,  /* STYLE_COLUMN */
@@ -555,30 +544,11 @@ bmbl_tree_expand(GtkTreeView * tree_view, GtkTreeIter * iter,
     gtk_tree_model_get(model, iter, MBNODE_COLUMN, &mbnode, -1);
     balsa_mailbox_node_scan_children(mbnode);
 
-    if (!mbnode->mailbox) {
-#if GTK_CHECK_VERSION(2, 8, 0)
+    if (!mbnode->mailbox)
         gtk_tree_store_set(GTK_TREE_STORE(model), iter,
                            ICON_COLUMN,
                            balsa_icon_id(BALSA_PIXMAP_MBOX_DIR_OPEN),
                            -1);
-#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
-        GdkPixbuf *pixbuf =
-            gtk_widget_render_icon(GTK_WIDGET(balsa_app.main_window),
-                                   BALSA_PIXMAP_MBOX_DIR_OPEN,
-                                   GTK_ICON_SIZE_MENU, NULL);
-        gtk_tree_store_set(GTK_TREE_STORE(model), iter,
-                           ICON_COLUMN, pixbuf,
-                           -1);
-        if(pixbuf) g_object_unref(pixbuf);
-        else { 
-            static int p=1;
-            if(p){
-                printf("icon %s not found.\n", BALSA_PIXMAP_MBOX_DIR_OPEN);
-                p = 0;
-            }
-        }
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
-    }
     g_object_unref(mbnode);
 
     if (gtk_tree_model_iter_children(model, &child_iter, iter)) {
@@ -646,30 +616,11 @@ bmbl_tree_collapse(GtkTreeView * tree_view, GtkTreeIter * iter,
 
     gtk_tree_model_get(model, iter, MBNODE_COLUMN, &mbnode, -1);
 
-    if (!mbnode->mailbox) {
-#if GTK_CHECK_VERSION(2, 8, 0)
+    if (!mbnode->mailbox)
         gtk_tree_store_set(GTK_TREE_STORE(model), iter,
                            ICON_COLUMN, 
                            balsa_icon_id(BALSA_PIXMAP_MBOX_DIR_CLOSED),
                            -1);
-#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
-        GdkPixbuf *pixbuf =
-            gtk_widget_render_icon(GTK_WIDGET(balsa_app.main_window),
-                                   BALSA_PIXMAP_MBOX_DIR_CLOSED,
-                                   GTK_ICON_SIZE_MENU, NULL);
-        gtk_tree_store_set(GTK_TREE_STORE(model), iter,
-                           ICON_COLUMN, pixbuf,
-                           -1);
-        if(pixbuf) g_object_unref(pixbuf);
-        else { 
-            static int p=1;
-            if(p){
-                printf("icon %s not found.\n", BALSA_PIXMAP_MBOX_DIR_CLOSED);
-                p = 0;
-            }
-        }
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
-    }
     g_object_unref(mbnode);
 
     bmbl_tree_collapse_helper(model, iter);
@@ -802,12 +753,7 @@ bmbl_do_popup(GtkTreeView * tree_view, GtkTreePath * path,
 
     menu = balsa_mailbox_node_get_context_menu(mbnode);
     g_object_ref(menu);
-#if GLIB_CHECK_VERSION(2, 10, 0)
     g_object_ref_sink(menu);
-#else                           /* GLIB_CHECK_VERSION(2, 10, 0) */
-    g_object_ref(menu);
-    gtk_object_sink(GTK_OBJECT(menu));
-#endif                          /* GLIB_CHECK_VERSION(2, 10, 0) */
     gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
 		   event_button, event_time);
     g_object_unref(menu);
@@ -1335,9 +1281,6 @@ bmbl_store_redraw_mbnode(GtkTreeIter * iter, BalsaMailboxNode * mbnode)
     const gchar *icon;
     const gchar *name;
     gchar *tmp = NULL;
-#if !GTK_CHECK_VERSION(2, 8, 0)
-    GdkPixbuf *pixbuf;
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
     gboolean expose = FALSE;
 
     g_return_val_if_fail(mbnode, FALSE);
@@ -1412,7 +1355,6 @@ bmbl_store_redraw_mbnode(GtkTreeIter * iter, BalsaMailboxNode * mbnode)
         name = tmp = g_path_get_basename(mbnode->name);
     }
 
-#if GTK_CHECK_VERSION(2, 8, 0)
     gtk_tree_store_set(balsa_app.mblist_tree_store, iter,
                        MBNODE_COLUMN, mbnode,
                        ICON_COLUMN, balsa_icon_id(icon),
@@ -1422,25 +1364,6 @@ bmbl_store_redraw_mbnode(GtkTreeIter * iter, BalsaMailboxNode * mbnode)
                        UNREAD_COLUMN, "",
                        TOTAL_COLUMN,  "",
                        -1);
-#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
-    /* render icon may fail if the installation was incomplete */
-    pixbuf = gtk_widget_render_icon(GTK_WIDGET(balsa_app.main_window),
-                                    icon, GTK_ICON_SIZE_MENU, NULL);
-    gtk_tree_store_set(balsa_app.mblist_tree_store, iter,
-                       MBNODE_COLUMN, mbnode,
-                       ICON_COLUMN, pixbuf,
-                       NAME_COLUMN,   name,
-                       WEIGHT_COLUMN, PANGO_WEIGHT_NORMAL,
-                       STYLE_COLUMN, PANGO_STYLE_NORMAL,
-                       UNREAD_COLUMN, "",
-                       TOTAL_COLUMN,  "",
-                       -1);
-    if(pixbuf) g_object_unref(pixbuf);
-    else { 
-        static int p=1;
-        if(p){ printf("icon %s not found.\n", icon); p = 0; }
-    }
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
     g_free(tmp);
 
     if (mbnode->mailbox) {
@@ -1531,9 +1454,6 @@ bmbl_node_style(GtkTreeModel * model, GtkTreeIter * iter)
         const gchar *name;
         gchar *tmp = NULL;
         PangoWeight weight;
-#if !GTK_CHECK_VERSION(2, 8, 0)
-        GdkPixbuf *pixbuf;
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
 
         /* Set the style appropriate for unread_messages; we do this
          * even if the state hasn't changed, because we might be
@@ -1561,26 +1481,11 @@ bmbl_node_style(GtkTreeModel * model, GtkTreeIter * iter)
             mbnode->style &= ~MBNODE_STYLE_NEW_MAIL;
         }
 
-#if GTK_CHECK_VERSION(2, 8, 0)
         gtk_tree_store_set(GTK_TREE_STORE(model), iter,
                            ICON_COLUMN, balsa_icon_id(icon),
                            NAME_COLUMN, name,
                            WEIGHT_COLUMN, weight,
                            -1);
-#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
-        pixbuf = gtk_widget_render_icon(GTK_WIDGET(balsa_app.main_window),
-                                        icon, GTK_ICON_SIZE_MENU, NULL);
-        gtk_tree_store_set(GTK_TREE_STORE(model), iter,
-                           ICON_COLUMN, pixbuf,
-                           NAME_COLUMN, name,
-                           WEIGHT_COLUMN, weight,
-                           -1);
-        if(pixbuf) g_object_unref(pixbuf);
-        else { 
-            static int p=1;
-            if(p){ printf("icon %s not found.\n", icon); p = 0; }
-        }
-#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
         g_free(tmp);
 
     }
@@ -2089,21 +1994,12 @@ bmbl_mru_combo_box_setup(GtkComboBox * combo_box)
     BalsaMBListMRUOption *mro =
         g_object_get_data(G_OBJECT(combo_box), "mro");
     GList *list;
-#if GTK_CHECK_VERSION(2, 6, 0)
     GtkListStore *store;
     GtkTreeIter iter;
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    gint i;
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
 
     gtk_combo_box_set_active(combo_box, -1);
-#if GTK_CHECK_VERSION(2, 6, 0)
     store = GTK_LIST_STORE(gtk_combo_box_get_model(combo_box));
     gtk_list_store_clear(store);
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    for (i = g_slist_length(mro->real_urls) + 1; --i >= 0;)
-        gtk_combo_box_remove_text(combo_box, i);
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
     g_slist_foreach(mro->real_urls, (GFunc) g_free, NULL);
     g_slist_free(mro->real_urls);
     mro->real_urls = NULL;
@@ -2113,20 +2009,14 @@ bmbl_mru_combo_box_setup(GtkComboBox * combo_box)
         LibBalsaMailbox *mailbox;
 
         if ((mailbox = balsa_find_mailbox_by_url(url)) || !*url) {
-#if GTK_CHECK_VERSION(2, 6, 0)
 	    gtk_list_store_append(store, &iter);
 	    gtk_list_store_set(store, &iter,
                                0, mailbox ? mailbox->name : "",
                                1, FALSE, -1);
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-            gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box),
-                                      mailbox ? mailbox->name : "");
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
             mro->real_urls = g_slist_append(mro->real_urls, g_strdup(url));
         }
     }
 
-#if GTK_CHECK_VERSION(2, 6, 0)
     /* Separator: */
     gtk_list_store_append(store, &iter);
     gtk_list_store_set(store, &iter, 1, TRUE, -1);
@@ -2134,9 +2024,6 @@ bmbl_mru_combo_box_setup(GtkComboBox * combo_box)
     gtk_list_store_set(store, &iter,
                        0, _("Other..."),
 		       1, FALSE, -1);
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box), _("Other..."));
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
     gtk_combo_box_set_active(combo_box, 0);
 }
 
@@ -2191,7 +2078,6 @@ bmbl_mru_combo_box_destroy_cb(BalsaMBListMRUOption * mro)
  * Adds a last entry that pops up the whole mailbox tree. When an item
  * is clicked, the url_list is updated.
  */
-#if GTK_CHECK_VERSION(2, 6, 0)
 static gboolean
 bmbl_mru_separator_func(GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
 {
@@ -2201,21 +2087,17 @@ bmbl_mru_separator_func(GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
 
     return is_sep;
 }
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
 
 GtkWidget *
 balsa_mblist_mru_option_menu(GtkWindow * window, GList ** url_list)
 {
     GtkWidget *combo_box;
     BalsaMBListMRUOption *mro;
-#if GTK_CHECK_VERSION(2, 6, 0)
     GtkListStore *store;
     GtkCellRenderer *renderer;
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
 
     g_return_val_if_fail(url_list != NULL, NULL);
 
-#if GTK_CHECK_VERSION(2, 6, 0)
     store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_BOOLEAN);
     combo_box = gtk_combo_box_new_with_model(GTK_TREE_MODEL(store));
     renderer = gtk_cell_renderer_text_new();
@@ -2225,9 +2107,6 @@ balsa_mblist_mru_option_menu(GtkWindow * window, GList ** url_list)
     gtk_combo_box_set_row_separator_func(GTK_COMBO_BOX(combo_box),
                                          bmbl_mru_separator_func, NULL,
                                          NULL);
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-    combo_box = gtk_combo_box_new_text();
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
     mro = g_new(BalsaMBListMRUOption, 1);
 
     mro->window = window;
