@@ -19,14 +19,17 @@
  * 02111-1307, USA.
  */
 
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "balsa-mime-widget-callbacks.h"
+
 #include <string.h>
-#include "config.h"
 #include "balsa-app.h"
 #include <glib/gi18n.h>
 #include "libbalsa-vfs.h"
 #include "balsa-message.h"
 #include "balsa-mime-widget.h"
-#include "balsa-mime-widget-callbacks.h"
 
 #include <gdk/gdkkeysyms.h>
 
@@ -147,10 +150,12 @@ balsa_mime_widget_ctx_menu_save(GtkWidget * parent_widget,
                                             LIBBALSA_MESSAGE_BODY_UNSAFE,
                                             mime_body->body_type ==
                                             LIBBALSA_MESSAGE_BODY_TYPE_TEXT,
-                                            &err))
+                                            &err)) {
 	    balsa_information(LIBBALSA_INFORMATION_ERROR,
 			      _("Could not save %s: %s"),
 			      file_uri, err ? err->message : "Unknown error");
+            g_clear_error(&err);
+        }
     }
 
     g_object_unref(save_file);
@@ -230,9 +235,13 @@ balsa_mime_widget_key_press_event(GtkWidget * widget, GdkEventKey * event,
         if (event->state & GDK_SHIFT_MASK) {
 	    GtkWidget *current_widget = balsa_message_current_part_widget(bm);
 
-	    if (current_widget)
-		g_signal_emit_by_name(current_widget, "popup-menu");
-	    else
+	    if (current_widget) {
+                gboolean retval;
+
+                g_signal_emit_by_name(current_widget, "popup-menu", &retval);
+
+                return retval;
+            } else
 		return FALSE;
         } else
             return FALSE;
