@@ -19,9 +19,13 @@
  * 02111-1307, USA.
  */
 
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "balsa-mime-widget-text.h"
+
 #include <string.h>
 #include <stdlib.h>
-#include "config.h"
 #include "balsa-app.h"
 #include "html.h"
 #include <glib/gi18n.h>
@@ -30,13 +34,8 @@
 #include "store-address.h"
 #include "balsa-mime-widget.h"
 #include "balsa-mime-widget-callbacks.h"
-#include "balsa-mime-widget-text.h"
 #include "balsa-cite-bar.h"
 
-#if !GTK_CHECK_VERSION(2, 14, 0)
-#include <libgnome/gnome-url.h>
-#endif
-
 #if HAVE_GTKSOURCEVIEW
 #include <gtksourceview/gtksourceview.h>
 #include <gtksourceview/gtksourcebuffer.h>
@@ -177,8 +176,10 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
     if(g_ascii_strcasecmp(content_type, "text/x-vcard") == 0 ||
        g_ascii_strcasecmp(content_type, "text/directory") == 0) {
         mw = bm_widget_new_vcard(bm, mime_body, ptr, alloced);
-        if (mw)
+        if (mw) {
+            g_free(ptr);
             return mw;
+        }
         /* else it was not a vCard with at least one address; we'll just
          * show it as if it were text/plain. */
     }
@@ -928,11 +929,7 @@ handle_url(const message_url_t* url)
         gtk_statusbar_push(statusbar, context_id, notice);
         SCHEDULE_BAR_REFRESH();
         g_free(notice);
-#if GTK_CHECK_VERSION(2, 14, 0)
         gtk_show_uri(NULL, url->url, gtk_get_current_event_time(), &err);
-#else
-    gnome_url_show(url->url, &err);
-#endif
         if (err) {
             balsa_information(LIBBALSA_INFORMATION_WARNING,
                     _("Error showing %s: %s\n"), url->url,
@@ -1178,12 +1175,7 @@ balsa_gtk_html_popup(GtkWidget * html, BalsaMessage * bm)
     gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 
     gtk_widget_show_all(menu);
-#if GLIB_CHECK_VERSION(2, 10, 0)
     g_object_ref_sink(menu);
-#else                           /* GLIB_CHECK_VERSION(2, 10, 0) */
-    g_object_ref(menu);
-    gtk_object_sink(GTK_OBJECT(menu));
-#endif                          /* GLIB_CHECK_VERSION(2, 10, 0) */
     gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
                    0, gtk_get_current_event_time());
     g_object_unref(menu);
@@ -1203,11 +1195,7 @@ bm_widget_link_clicked(const gchar *url)
 {
     GError *err = NULL;
 
-#if GTK_CHECK_VERSION(2, 14, 0)
     gtk_show_uri(NULL, url, gtk_get_current_event_time(), &err);
-#else
-    gnome_url_show(url, &err);
-#endif
     if (err) {
         balsa_information(LIBBALSA_INFORMATION_WARNING,
                 _("Error showing %s: %s\n"), url, err->message);
@@ -1291,6 +1279,5 @@ bm_widget_new_vcard(BalsaMessage *bm, LibBalsaMessageBody *mime_body,
         
     g_object_set_data(G_OBJECT(mw->widget), "mime-body", mime_body);
     gtk_widget_show_all(mw->widget);
-    g_free(ptr);
     return mw;
 }
