@@ -19,13 +19,16 @@
  * 02111-1307, USA.
  */
 
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "balsa-print-object-header.h"
+
 #include <gtk/gtk.h>
-#include "config.h"
 #include <glib/gi18n.h>
 #include "balsa-print-object.h"
 #include "balsa-print-object-decor.h"
 #include "balsa-print-object-default.h"
-#include "balsa-print-object-header.h"
 
 
 /* object related functions */
@@ -224,16 +227,12 @@ balsa_print_object_header_new_real(GList * list,
 	gint prot = libbalsa_message_body_protection(sig_body);
 
 	if ((prot & LIBBALSA_PROTECT_SIGN) &&
-	    (prot & (LIBBALSA_PROTECT_RFC3156 | LIBBALSA_PROTECT_SMIMEV3)))
-	{
+	    (prot & (LIBBALSA_PROTECT_RFC3156 | LIBBALSA_PROTECT_SMIMEV3))) {
 	    GMimeGpgmeSigstat *siginfo = sig_body->parts->next->sig_info;
-	    gchar *sig_status;
 
-	    sig_status =
-		g_strconcat
-		(libbalsa_gpgme_sig_protocol_name(siginfo->protocol),
-		 libbalsa_gpgme_sig_stat_to_gchar(siginfo->status), NULL);
-	    g_string_append_printf(header_buf, "%s\n", sig_status);
+	    g_string_append_printf(header_buf, "%s%s\n",
+                                   libbalsa_gpgme_sig_protocol_name(siginfo->protocol),
+                                   libbalsa_gpgme_sig_stat_to_gchar(siginfo->status));
 	}
     }
 #endif				/* HAVE_GPGME */
