@@ -19,15 +19,20 @@
  * 02111-1307, USA.
  */
 
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "balsa-print-object-text.h"
+
 #include <gtk/gtk.h>
 #include <string.h>
-#include "config.h"
 #include <glib/gi18n.h>
+#include "libbalsa.h"
+#include "rfc2445.h"
 #include "balsa-icons.h"
 #include "balsa-print-object.h"
 #include "balsa-print-object-decor.h"
 #include "balsa-print-object-default.h"
-#include "balsa-print-object-text.h"
 
 
 typedef enum {
@@ -423,9 +428,6 @@ balsa_print_object_text(GList *list, GtkPrintContext * context,
 
 /* note: a vcard is an icon plus a series of labels/text, so this function actually
  * returns a BalsaPrintObjectDefault... */
-#ifdef HAVE_GNOME
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
-#endif
 
 #define ADD_VCARD_FIELD(buf, labwidth, layout, field, descr)		\
     do {								\
@@ -452,7 +454,7 @@ balsa_print_object_text_vcard(GList * list,
     PangoTabArray *tabs;
     GString *desc_buf;
     gdouble c_max_height;
-    LibBalsaAddress * addr;
+    LibBalsaAddress * addr = NULL;
     gchar *textbuf;
 
     /* check if we can create an address from the body and fall back to default if 
@@ -461,10 +463,11 @@ balsa_print_object_text_vcard(GList * list,
 	textbuf = g_strdup(body->buffer);
     else
 	libbalsa_message_body_get_content(body, &textbuf, NULL);
-    addr = libbalsa_address_new_from_vcard(textbuf, body->charset);
+    if (textbuf)
+        addr = libbalsa_address_new_from_vcard(textbuf, body->charset);
     if (!addr) {
 	g_free(textbuf);
-	return balsa_print_object_default(list, context, body, psetup);
+	return balsa_print_object_text(list, context, body, psetup);
     }
 
     /* proceed with the address information */
@@ -555,6 +558,216 @@ balsa_print_object_text_vcard(GList * list,
 }
 
 
+/* add a text/calendar object */
+
+#define ADD_VCAL_FIELD(buf, labwidth, layout, field, descr)		\
+    do {								\
+	if (field) {							\
+	    gint label_width = p_string_width_from_layout(layout, descr); \
+	    if (label_width > labwidth)					\
+		labwidth = label_width;					\
+	    if ((buf)->len > 0)						\
+		g_string_append_c(buf, '\n');				\
+	    g_string_append_printf(buf, "%s\t%s", descr, field);	\
+	}								\
+    } while(0)
+
+#define ADD_VCAL_DATE(buf, labwidth, layout, date, descr)               \
+    do {                                                                \
+        if (date != (time_t) -1) {                                      \
+            gchar * _dstr =                                             \
+                libbalsa_date_to_utf8(&date, balsa_app.date_string);    \
+            ADD_VCAL_FIELD(buf, labwidth, layout, _dstr, descr);        \
+            g_free(_dstr);                                              \
+        }                                                               \
+    } while (0)
+
+#define ADD_VCAL_ADDRESS(buf, labwidth, layout, addr, descr)            \
+    do {                                                                \
+        if (addr) {                                                     \
+            gchar * _astr = libbalsa_vcal_attendee_to_str(addr);        \
+            ADD_VCAL_FIELD(buf, labwidth, layout, _astr, descr);        \
+            g_free(_astr);                                              \
+        }                                                               \
+    } while (0)
+
+
+GList *
+balsa_print_object_text_calendar(GList * list,
+                                 GtkPrintContext * context,
+                                 LibBalsaMessageBody * body,
+                                 BalsaPrintSetup * psetup)
+{
+    BalsaPrintObjectDefault *pod;
+    BalsaPrintObject *po;
+    PangoFontDescription *header_font;
+    PangoLayout *test_layout;
+    PangoTabArray *tabs;
+    GString *desc_buf;
+    LibBalsaVCal * vcal_obj;
+    GList * this_ev;
+    guint first_page;
+    GList *par_parts;
+    GList *this_par_part;
+    gdouble c_at_y;
+
+    /* check if we can evaluate the body as calendar object and fall back
+     * to text if not */
+    if (!(vcal_obj = libbalsa_vcal_new_from_body(body)))
+	return balsa_print_object_text(list, context, body, psetup);
+
+    /* proceed with the address information */
+    pod = g_object_new(BALSA_TYPE_PRINT_OBJECT_DEFAULT, NULL);
+    g_assert(pod != NULL);
+    po = BALSA_PRINT_OBJECT(pod);
+
+    /* create the part */
+    po->depth = psetup->curr_depth;
+    po->c_width =
+	psetup->c_width - 2 * psetup->curr_depth * C_LABEL_SEP;
+
+    /* get the stock calendar icon or the mime type icon on fail */
+    pod->pixbuf =
+	gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
+				 "stock_calendar", 48,
+				 GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
+    if (!pod->pixbuf) {
+	gchar *conttype = libbalsa_message_body_get_mime_type(body);
+	
+	pod->pixbuf = libbalsa_icon_finder(conttype, NULL, NULL, GTK_ICON_SIZE_DND);
+    }
+    pod->c_image_width = gdk_pixbuf_get_width(pod->pixbuf);
+    pod->c_image_height = gdk_pixbuf_get_height(pod->pixbuf);
+
+    /* move to the next page if the icon doesn't fit */
+    if (psetup->c_y_pos + pod->c_image_height > psetup->c_height) {
+	psetup->c_y_pos = 0;
+	psetup->page_count++;
+    }
+
+    /* create a layout for calculating the maximum label width and for splitting
+     * the body (if necessary) */
+    header_font =
+	pango_font_description_from_string(balsa_app.print_header_font);
+    test_layout = gtk_print_context_create_pango_layout(context);
+    pango_layout_set_font_description(test_layout, header_font);
+    pango_font_description_free(header_font);
+
+    /* add fields from the events*/
+    desc_buf = g_string_new("");
+    pod->p_label_width = 0;
+    for (this_ev = vcal_obj->vevent; this_ev; this_ev = g_list_next(this_ev)) {
+        LibBalsaVEvent * event = (LibBalsaVEvent *) this_ev->data;
+
+        if (desc_buf->len > 0)
+            g_string_append_c(desc_buf, '\n');
+        ADD_VCAL_FIELD(desc_buf, pod->p_label_width, test_layout,
+                       event->summary, _("Summary"));
+        ADD_VCAL_ADDRESS(desc_buf, pod->p_label_width, test_layout,
+                         event->organizer, _("Organizer"));
+        ADD_VCAL_DATE(desc_buf, pod->p_label_width, test_layout,
+                      event->start, _("Start"));
+        ADD_VCAL_DATE(desc_buf, pod->p_label_width, test_layout,
+                      event->end, _("End"));
+        ADD_VCAL_FIELD(desc_buf, pod->p_label_width, test_layout,
+                       event->location, _("Location"));
+        if (event->attendee) {
+            GList * att = event->attendee;
+            gchar * this_att;
+
+            this_att =
+                libbalsa_vcal_attendee_to_str(LIBBALSA_ADDRESS(att->data));
+            att = g_list_next(att);
+            ADD_VCAL_FIELD(desc_buf, pod->p_label_width, test_layout,
+                           this_att, att ? _("Attendees") : _("Attendee"));
+            g_free(this_att);
+            for (; att; att = g_list_next(att)) {
+                this_att =
+                    libbalsa_vcal_attendee_to_str(LIBBALSA_ADDRESS(att->data));
+                g_string_append_printf(desc_buf, "\n\t%s", this_att);
+                g_free(this_att);
+            }
+        }
+        if (event->description) {
+            gchar ** desc_lines = g_strsplit(event->description, "\n", -1);
+            gint i;
+
+            ADD_VCAL_FIELD(desc_buf, pod->p_label_width, test_layout,
+                           desc_lines[0], _("Description"));
+            for (i = 1; desc_lines[i]; i++)
+                g_string_append_printf(desc_buf, "\n\t%s", desc_lines[i]);
+            g_strfreev(desc_lines);
+        }
+    }
+    g_object_unref(vcal_obj);
+
+    /* add a small space between label and value */
+    pod->p_label_width += C_TO_P(C_LABEL_SEP);
+
+    /* configure the layout so we can split the text */
+    pango_layout_set_indent(test_layout, -pod->p_label_width);
+    tabs =
+	pango_tab_array_new_with_positions(1, FALSE, PANGO_TAB_LEFT,
+					   pod->p_label_width);
+    pango_layout_set_tabs(test_layout, tabs);
+    pango_tab_array_free(tabs);
+    pango_layout_set_width(test_layout,
+			   C_TO_P(po->c_width -
+				  4 * C_LABEL_SEP - pod->c_image_width));
+    pango_layout_set_alignment(test_layout, PANGO_ALIGN_LEFT);
+
+    /* split paragraph if necessary */
+    first_page = psetup->page_count - 1;
+    c_at_y = psetup->c_y_pos;
+    par_parts =
+        split_for_layout(test_layout, desc_buf->str, NULL, psetup, TRUE, NULL);
+    g_string_free(desc_buf, TRUE);
+
+    /* set the parameters of the first part */
+    pod->description = (gchar *) par_parts->data;
+    pod->c_text_height =
+	P_TO_C(p_string_height_from_layout(test_layout, pod->description));
+    po->on_page = first_page++;
+    po->c_at_x = psetup->c_x0 + po->depth * C_LABEL_SEP;
+    po->c_at_y = psetup->c_y0 + c_at_y;
+    po->c_height = MAX(pod->c_image_height, pod->c_text_height);
+    list = g_list_append(list, pod);
+
+    /* add more parts */
+    for (this_par_part = g_list_next(par_parts); this_par_part;
+         this_par_part = g_list_next(this_par_part)) {
+        BalsaPrintObjectDefault * new_pod;
+        BalsaPrintObject *new_po;
+        
+        /* create a new object */
+        new_pod = g_object_new(BALSA_TYPE_PRINT_OBJECT_DEFAULT, NULL);
+        g_assert(new_pod != NULL);
+        new_po = BALSA_PRINT_OBJECT(new_pod);
+
+        /* fill data */
+        new_pod->p_label_width = pod->p_label_width;
+        new_pod->c_image_width = pod->c_image_width;
+        new_pod->description = (gchar *) this_par_part->data;
+        new_pod->c_text_height =
+            P_TO_C(p_string_height_from_layout(test_layout, new_pod->description));
+        new_po->on_page = first_page++;
+        new_po->c_at_x = psetup->c_x0 + po->depth * C_LABEL_SEP;
+        new_po->c_at_y = psetup->c_y0;
+        new_po->c_height = new_pod->c_text_height;
+        new_po->depth = psetup->curr_depth;
+        new_po->c_width =
+            psetup->c_width - 2 * psetup->curr_depth * C_LABEL_SEP;
+        
+        /* append */
+        list = g_list_append(list, new_pod);
+    }
+    g_list_free(par_parts);
+    g_object_unref(G_OBJECT(test_layout));
+
+    return list;
+}
+
+
 static void
 balsa_print_object_text_draw(BalsaPrintObject * self,
 			     GtkPrintContext * context,
