@@ -23,9 +23,10 @@
  * Callbacks for the filter edit dialog
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 
-#define _XOPEN_SOURCE /* glibc2 needs this */
 /* yellow dog has crappy libc and needs pthread.h to be included here */
 #ifdef BALSA_USE_THREADS
 #  include <pthread.h>
@@ -42,12 +43,7 @@
 #include "mailbox-filter.h"
 #include <glib/gi18n.h>
 #include "libbalsa-conf.h"
-
-#if !GTK_CHECK_VERSION(2, 14, 0)
-#ifdef HAVE_GNOME
-#include <gnome.h>
-#endif
-#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
+#include "missing.h"
 
 /* Defined in filter-edit-dialog.c*/
 extern option_list fe_search_type[4];
@@ -125,9 +121,19 @@ extern GtkWidget *fe_popup_entry;
 /* action field */
 extern GtkWidget *fe_action_option_menu;
 
+/* action list */
+extern option_list fe_actions[];
+
 /* Mailboxes option menu */
 extern GtkWidget * fe_mailboxes;
 
+/* Colorize widgets */
+extern GtkWidget * fe_color_buttons;
+extern GtkWidget * fe_foreground_set;
+extern GtkWidget * fe_foreground;
+extern GtkWidget * fe_background_set;
+extern GtkWidget * fe_background;
+
 /* Different buttons that need to be greyed or ungreyed */
 extern GtkWidget * fe_new_button;
 extern GtkWidget * fe_delete_button;
@@ -820,9 +826,7 @@ condition_dialog_response(GtkWidget * dialog, gint response,
                           gpointer throwaway)
 {
     LibBalsaCondition *new_cnd;
-#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
-    GError *err;
-#endif
+    GError *err = NULL;
 
     switch (response) {
     case GTK_RESPONSE_OK:       /* OK button */
@@ -878,14 +882,8 @@ condition_dialog_response(GtkWidget * dialog, gint response,
         gtk_widget_hide(dialog);
         break;
     case GTK_RESPONSE_HELP:     /* Help button */
-#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
-	err = NULL;
-#if GTK_CHECK_VERSION(2, 14, 0)
         gtk_show_uri(NULL, "ghelp:balsa?win-condition",
                      gtk_get_current_event_time(), &err);
-#else                           /* GTK_CHECK_VERSION(2, 14, 0) */
-	gnome_help_display("balsa", "win-condition", &err);
-#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
 	if (err) {
 	    balsa_information_parented(GTK_WINDOW(dialog),
 		    LIBBALSA_INFORMATION_WARNING,
@@ -893,7 +891,6 @@ condition_dialog_response(GtkWidget * dialog, gint response,
 		    err->message);
 	    g_error_free(err);
 	}
-#endif
 	break;
     }
     gtk_widget_set_sensitive(fe_window, TRUE);
@@ -1449,9 +1446,7 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
         gtk_tree_view_get_model(fe_filters_list);
     GtkTreeIter iter;
     gboolean valid;
-#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
-    GError *err;
-#endif
+    GError *err = NULL;
     
     switch (response) {
     case GTK_RESPONSE_OK:       /* OK button */
@@ -1490,14 +1485,8 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
         break;
 
     case GTK_RESPONSE_HELP:     /* Help button */
-#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
-	err = NULL;
-#if GTK_CHECK_VERSION(2, 14, 0)
         gtk_show_uri(NULL, "ghelp:balsa?win-filters",
                      gtk_get_current_event_time(), &err);
-#else                           /* GTK_CHECK_VERSION(2, 14, 0) */
-	gnome_help_display("balsa", "win-filters", &err);
-#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
 	if (err) {
 	    balsa_information_parented(GTK_WINDOW(dialog),
 		    LIBBALSA_INFORMATION_WARNING,
@@ -1505,7 +1494,6 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
 		    err->message);
 	    g_error_free(err);
 	}
-#endif
 	break;
 
     default:
@@ -1524,8 +1512,15 @@ fe_action_selected(GtkWidget * widget, gpointer data)
 {
     FilterActionType type =
         (FilterActionType) fe_combo_box_get_value(widget);
-    gtk_widget_set_sensitive(GTK_WIDGET(fe_mailboxes),
-                             type != FILTER_TRASH);
+    if (type == FILTER_COLOR) {
+        gtk_widget_hide(fe_mailboxes);
+        gtk_widget_show_all(fe_color_buttons);
+    } else {
+        gtk_widget_hide(fe_color_buttons);
+        gtk_widget_show(fe_mailboxes);
+        gtk_widget_set_sensitive(GTK_WIDGET(fe_mailboxes),
+                                 type != FILTER_TRASH);
+    }
     set_button_sensitivities(TRUE);
 }                       /* end fe_action_selected() */
 
@@ -1780,9 +1775,6 @@ fe_delete_pressed(GtkWidget * widget, gpointer data)
         gtk_entry_set_text(GTK_ENTRY(fe_name_entry),"");
         gtk_entry_set_text(GTK_ENTRY(fe_popup_entry),"");
         /*gtk_option_menu_set_history(GTK_OPTION_MENU(fe_mailboxes), 0); */
-#if !GTK_CHECK_VERSION(2, 6, 0)
-        gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(fe_sound_entry))),"");
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
         gtk_list_store_clear(GTK_LIST_STORE
                              (gtk_tree_view_get_model
                               (fe_conditions_list)));
@@ -1895,7 +1887,36 @@ fe_apply_pressed(GtkWidget * widget, gpointer data)
 
     fil->action=action;
 
-    if (fil->action!=FILTER_TRASH)
+    if (fil->action == FILTER_COLOR) {
+        GdkColor color;
+        GString *string = g_string_new(NULL);
+        GtkToggleButton *toggle_button;
+        gchar *color_string;
+
+        toggle_button = (GTK_TOGGLE_BUTTON(fe_foreground_set));
+        if (gtk_toggle_button_get_active(toggle_button)) {
+            gtk_color_button_get_color(GTK_COLOR_BUTTON(fe_foreground),
+                                       &color);
+            color_string = gdk_color_to_string(&color);
+            g_string_append_printf(string, "foreground:%s", color_string);
+            g_free(color_string);
+            gtk_toggle_button_set_active(toggle_button, FALSE);
+        }
+
+        toggle_button = (GTK_TOGGLE_BUTTON(fe_background_set));
+        if (gtk_toggle_button_get_active(toggle_button)) {
+            gtk_color_button_get_color(GTK_COLOR_BUTTON(fe_background),
+                                       &color);
+            color_string = gdk_color_to_string(&color);
+            if (string->len > 0)
+                g_string_append_c(string, ';');
+            g_string_append_printf(string, "background:%s", color_string);
+            g_free(color_string);
+            gtk_toggle_button_set_active(toggle_button, FALSE);
+        }
+
+        fil->action_string = g_string_free(string, FALSE);
+    } else if (fil->action!=FILTER_TRASH)
         fil->action_string=g_strdup(balsa_mblist_mru_option_menu_get(fe_mailboxes));
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fe_popup_button))) {
         static gchar defstring[] = N_("Filter has matched");
@@ -1912,13 +1933,8 @@ fe_apply_pressed(GtkWidget * widget, gpointer data)
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fe_sound_button))) {
         gchar *tmpstr;
         
-#if GTK_CHECK_VERSION(2, 6, 0)
         tmpstr = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER
                                                (fe_sound_entry));
-#else /* GTK_CHECK_VERSION(2, 6, 0) */
-        tmpstr =
-            gtk_editable_get_chars(GTK_EDITABLE(fe_sound_entry), 0, -1);
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
         if ((!tmpstr) || (tmpstr[0] == '\0')) {
             g_free(tmpstr);
             libbalsa_filter_free(fil, GINT_TO_POINTER(TRUE));
@@ -2001,7 +2017,7 @@ fe_filters_list_selection_changed(GtkTreeSelection * selection,
     GtkTreeModel *model;
     GtkTreeIter iter;
     LibBalsaFilter* fil;
-    int pos;
+    int pos, i;
 
     if (!gtk_tree_selection_get_selected(selection, &model, &iter)) {
         if (gtk_tree_model_iter_n_children(model, NULL) == 0)
@@ -2020,21 +2036,40 @@ fe_filters_list_selection_changed(GtkTreeSelection * selection,
                        ? fil->popup_text : "");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_sound_button),
                                  fil->sound!=NULL);
-#if GTK_CHECK_VERSION(2, 6, 0)
     if (fil->sound)
         gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(fe_sound_entry),
                                       fil->sound);
-#else /* GTK_CHECK_VERSION(2, 6, 0) */
-    gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(fe_sound_entry))),
-                       fil->sound!=NULL ? fil->sound : "");
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
     
-    gtk_combo_box_set_active(GTK_COMBO_BOX(fe_action_option_menu),
-                             fil->action - 1);
+    for (i = 0; i < FILTER_N_TYPES - 1; i++)
+        if (((FilterActionType) fe_actions[i].value) == fil->action)
+            gtk_combo_box_set_active(GTK_COMBO_BOX(fe_action_option_menu),
+                                     i);
     pos = (fil->condition
            && fil->condition->type == CONDITION_AND) ? 1 : 0;
     gtk_combo_box_set_active(GTK_COMBO_BOX(fe_op_codes_option_menu), pos);
-    if (fil->action!=FILTER_TRASH && fil->action_string)
+    if (fil->action == FILTER_COLOR) {
+        gchar **parts, **p;
+        GdkColor color;
+
+        parts = g_strsplit(fil->action_string, ";", 2);
+        for (p = parts; *p; p++) {
+            if (g_str_has_prefix(*p, "foreground:")) {
+                gdk_color_parse((*p) + 11, &color);
+                gtk_color_button_set_color(GTK_COLOR_BUTTON(fe_foreground),
+                                           &color);
+                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+                                             (fe_foreground_set), TRUE);
+            }
+            if (g_str_has_prefix(*p, "background:")) {
+                gdk_color_parse((*p) + 11, &color);
+                gtk_color_button_set_color(GTK_COLOR_BUTTON(fe_background),
+                                           &color);
+                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+                                             (fe_background_set), TRUE);
+            }
+        }
+        g_strfreev(parts);
+    } else if (fil->action!=FILTER_TRASH && fil->action_string)
         balsa_mblist_mru_option_menu_set(fe_mailboxes,
                                          fil->action_string);
     /* We free the conditions */
@@ -2090,7 +2125,6 @@ fe_date_sample(void)
     return label;
 }                               /* end fe_date_sample */
 
-#if GTK_CHECK_VERSION(2, 6, 0)
 /* Callback for the sound file-chooser-button's dialog. */
 void
 fe_sound_response(GtkDialog * dialog, gint response)
@@ -2098,4 +2132,20 @@ fe_sound_response(GtkDialog * dialog, gint response)
     if (response == GTK_RESPONSE_ACCEPT)
         set_button_sensitivities(TRUE);
 }
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
+
+/* Callback for color check-buttons' "toggled" signal */
+void
+fe_color_check_toggled(GtkToggleButton * check_button, gpointer data)
+{
+    GtkWidget *color_button = GTK_WIDGET(data);
+    gtk_widget_set_sensitive(color_button,
+                             gtk_toggle_button_get_active(check_button));
+    set_button_sensitivities(TRUE);
+}
+
+/* Callback for color buttons' "color-set" signal */
+void
+fe_color_set(GtkColorButton * color_button, gpointer data)
+{
+    set_button_sensitivities(TRUE);
+}
