@@ -19,7 +19,9 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 
 #include <string.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
