@@ -39,17 +39,16 @@
    The mailbox_name field is displayed only for special mailboxes
    and POP3 mailboxes.
 */
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "mailbox-conf.h"
 
 #include <gtk/gtk.h>
-#if !GTK_CHECK_VERSION(2, 6, 0) && HAVE_GNOME
-#include <gnome.h>  /* needed by GnomeFileEntry */
-#endif
 #include <string.h>
 
 #include "balsa-app.h"
 #include "balsa-mblist.h"
-#include "mailbox-conf.h"
 #include "mailbox-node.h"
 #include "pref-manager.h"
 #include "save-restore.h"
@@ -83,12 +82,6 @@ struct _MailboxConfWindow {
     gboolean ok_sensitive;
 
     union {
-#if !GTK_CHECK_VERSION(2, 6, 0) && HAVE_GNOME
-	/* for local mailboxes */
-	struct local { 
-	    GnomeFileEntry *path;
-	} local;
-#endif /* GTK_CHECK_VERSION(2, 6, 0) */
 	/* for imap mailboxes & directories */
 	struct {
 	    GtkWidget *port;
@@ -984,13 +977,12 @@ mailbox_conf_add(MailboxConfWindow * mcw)
     if ( LIBBALSA_IS_MAILBOX_LOCAL(mcw->mailbox) ) {
 	LibBalsaMailboxLocal *ml  = LIBBALSA_MAILBOX_LOCAL(mcw->mailbox);
 	gchar *path;
-	int res;
 
         path =
             gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(mcw->window));
-	res = libbalsa_mailbox_local_set_path(ml, path, TRUE);
 
-	if( res != 0) {
+        if (libbalsa_mailbox_local_set_path(ml, path, TRUE) != 0) {
+            g_free(path);
 	    g_object_unref(G_OBJECT(mcw->mailbox));
 	    mcw->mailbox = NULL;
 	    return;
@@ -1001,6 +993,8 @@ mailbox_conf_add(MailboxConfWindow * mcw)
                                         balsa_app.local_mail_directory);
         printf("Save to config: %d\n", save_to_config);
 	mcw->mailbox->name = g_path_get_basename(path);
+        g_free(path);
+
 	balsa_mailbox_local_append(mcw->mailbox);
     }
     mbnode = balsa_mailbox_node_new_from_mailbox(mcw->mailbox);
