@@ -19,7 +19,10 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "mailbox-node.h"
 
 #include <unistd.h>
 #include <string.h>
@@ -28,7 +31,6 @@
 #include "folder-scanners.h"
 #include "folder-conf.h"
 #include "mailbox-conf.h"
-#include "mailbox-node.h"
 #include "save-restore.h"
 #include "filter.h"
 #include "imap-server.h"
@@ -712,6 +714,14 @@ balsa_mailbox_node_save_config(BalsaMailboxNode* mn, const gchar* group)
 /* ---------------------------------------------------------------------
  * Rescanning.
  * --------------------------------------------------------------------- */
+static BalsaMailboxNode *
+find_dir(const gchar * dir)
+{
+    if (strcmp(dir, balsa_app.local_mail_directory) == 0)
+        return g_object_ref(balsa_app.root_node);
+    return balsa_find_dir(NULL, dir);
+}
+
 void
 balsa_mailbox_local_append(LibBalsaMailbox* mbx)
 {
@@ -723,7 +733,7 @@ balsa_mailbox_local_append(LibBalsaMailbox* mbx)
 
     for(dir = g_strdup(libbalsa_mailbox_local_get_path(mbx));
         strlen(dir)>1 /* i.e dir != "/" */ &&
-            !(parent = balsa_find_dir(NULL, dir));
+            !(parent = find_dir(dir));
         ) {
         gchar* tmp =  g_path_get_dirname(dir);
         g_free(dir);
@@ -758,8 +768,8 @@ balsa_mailbox_node_rescan(BalsaMailboxNode * mn)
     } else {
         balsa_remove_children_mailbox_nodes(mn);
         mn = balsa_app.root_node;
-        mn->scanned = FALSE;
     }
+    mn->scanned = FALSE;
     balsa_mailbox_node_append_subtree(mn);
 }
 
