@@ -19,9 +19,11 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
-
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 #include "message-window.h"
+
 #include "balsa-app.h"
 #include "balsa-message.h"
 #include "balsa-icons.h"
@@ -62,9 +64,7 @@ static void next_message_cb            (GtkAction * action, gpointer data);
 static void previous_message_cb        (GtkAction * action, gpointer data);
 static void next_unread_cb             (GtkAction * action, gpointer);
 static void next_flagged_cb            (GtkAction * action, gpointer);
-#ifdef HAVE_GTK_PRINT
 static void page_setup_cb              (GtkAction * action, gpointer data);
-#endif
 static void print_cb                   (GtkAction * action, gpointer);
 static void trash_cb                   (GtkAction * action, gpointer);
 #ifdef HAVE_GTKHTML
@@ -235,10 +235,8 @@ static const GtkActionEntry entries[] = {
     {"ViewMenu", NULL, N_("_View")},
     {"MoveMenu", NULL, N_("M_ove")},
     {"MessageMenu", NULL, N_("_Message")},
-#ifdef HAVE_GTK_PRINT
     {"PageSetup", NULL, N_("Page _Setup"), "<control>S",
      N_("Set up page for printing"), G_CALLBACK(page_setup_cb)},
-#endif                          /* HAVE_GTK_PRINT */
     {"Print", GTK_STOCK_PRINT, N_("_Print..."), "<control>P",
      N_("Print current message"), G_CALLBACK(print_cb)},
     {"Close", GTK_STOCK_CLOSE, N_("_Close"), "<control>W",
@@ -329,9 +327,7 @@ static const char *ui_description =
 "<ui>"
 "  <menubar name='MainMenu'>"
 "    <menu action='FileMenu'>"
-#ifdef HAVE_GTK_PRINT
 "      <menuitem action='PageSetup'/>"
-#endif                          /* HAVE_GTK_PRINT */
 "      <menuitem action='Print'/>"
 "      <separator/>"
 "      <menuitem action='Close'/>"
@@ -522,7 +518,7 @@ message_window_new(LibBalsaMailbox * mailbox, guint msgno)
     BalsaToolbarModel *model;
     GtkUIManager *ui_manager;
     GtkAccelGroup *accel_group; 
-    GError *error;
+    GError *error = NULL;
     GtkWidget *menubar;
     GtkWidget *toolbar;
     GtkWidget *move_menu, *submenu;
@@ -560,7 +556,6 @@ message_window_new(LibBalsaMailbox * mailbox, guint msgno)
     gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
     g_object_unref(accel_group);
 
-    error = NULL;
     if (!gtk_ui_manager_add_ui_from_string
         (ui_manager, ui_description, -1, &error)) {
         g_message("building menus failed: %s", error->message);
@@ -672,7 +667,7 @@ destroy_message_window(GtkWidget * widget, MessageWindow * mw)
 static void
 mw_expunged_cb(LibBalsaMailbox * mailbox, guint msgno, MessageWindow * mw)
 {
-    if ((guint) mw->message->msgno == msgno)
+    if (mw->message && (guint) mw->message->msgno == msgno)
         gtk_widget_destroy(mw->window);
 }
 
@@ -935,7 +930,6 @@ next_flagged_cb(GtkAction * action, gpointer data)
 }
 
 
-#ifdef HAVE_GTK_PRINT
 static void
 page_setup_cb(GtkAction * action, gpointer data)
 {
@@ -943,7 +937,6 @@ page_setup_cb(GtkAction * action, gpointer data)
 
     message_print_page_setup(GTK_WINDOW(mw->window));
 }
-#endif
 
 
 static void
