@@ -18,16 +18,19 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
  * 02111-1307, USA.
  */
-#include "config.h"
 
-#define _XOPEN_SOURCE 500
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "save-restore.h"
 
 #include <stdlib.h>
 #include <string.h>
+#if HAVE_GNOME
 #include <gconf/gconf-client.h>
+#endif
 #include <glib/gi18n.h>
 #include "balsa-app.h"
-#include "save-restore.h"
 #include "server.h"
 #include "quote-color.h"
 #include "toolbar-prefs.h"
@@ -510,7 +513,6 @@ config_load_smtp_server(const gchar * key, const gchar * value, gpointer data)
 }
 #endif                          /* ENABLE_ESMTP */
 
-#ifdef HAVE_GTK_PRINT
 static gboolean
 load_gtk_print_setting(const gchar * key, const gchar * value, gpointer data)
 {
@@ -577,7 +579,6 @@ restore_gtk_page_setup()
     
     return page_setup;
 }
-#endif
 
 static gint
 config_global_load(void)
@@ -777,7 +778,11 @@ config_global_load(void)
     balsa_app.show_notebook_tabs = libbalsa_conf_get_bool("ShowTabs=false");
 
     /* ... alternative layout of main window */
-    balsa_app.alternative_layout = libbalsa_conf_get_bool("AlternativeLayout=false");
+    balsa_app.layout_type = libbalsa_conf_get_int("LayoutType=0");
+    if (balsa_app.layout_type != LAYOUT_DEFAULT && 
+        balsa_app.layout_type != LAYOUT_WIDE_MSG &&
+        balsa_app.layout_type != LAYOUT_WIDE_SCREEN)
+        balsa_app.layout_type = LAYOUT_DEFAULT;
     balsa_app.view_message_on_open = libbalsa_conf_get_bool("ViewMessageOnOpen=true");
     balsa_app.pgdownmod = libbalsa_conf_get_bool("PageDownMod=false");
     balsa_app.pgdown_percent = libbalsa_conf_get_int("PageDownPercent=50");
@@ -827,7 +832,6 @@ config_global_load(void)
     libbalsa_conf_push_group("Printing");
 
     /* ... Printing */
-#ifdef HAVE_GTK_PRINT
     if (balsa_app.page_setup)
 	g_object_unref(G_OBJECT(balsa_app.page_setup));
     balsa_app.page_setup = restore_gtk_page_setup();
@@ -835,27 +839,6 @@ config_global_load(void)
     balsa_app.margin_top = libbalsa_conf_get_double("TopMargin");
     balsa_app.margin_right = libbalsa_conf_get_double("RightMargin");
     balsa_app.margin_bottom = libbalsa_conf_get_double("BottomMargin");
-#else
-    g_free(balsa_app.paper_size);
-    balsa_app.paper_size =
-	libbalsa_conf_get_string("PaperSize=" DEFAULT_PAPER_SIZE);
-    g_free(balsa_app.margin_left);
-    balsa_app.margin_left   = libbalsa_conf_get_string("LeftMargin");
-    g_free(balsa_app.margin_top);
-    balsa_app.margin_top    = libbalsa_conf_get_string("TopMargin");
-    g_free(balsa_app.margin_right);
-    balsa_app.margin_right  = libbalsa_conf_get_string("RightMargin");
-    g_free(balsa_app.margin_bottom);
-    balsa_app.margin_bottom = libbalsa_conf_get_string("BottomMargin");
-    g_free(balsa_app.print_unit);
-    balsa_app.print_unit    = libbalsa_conf_get_string("PrintUnit");
-    g_free(balsa_app.print_layout);
-    balsa_app.print_layout  = libbalsa_conf_get_string("PrintLayout");
-    g_free(balsa_app.paper_orientation);
-    balsa_app.paper_orientation  = libbalsa_conf_get_string("PaperOrientation");
-    g_free(balsa_app.page_orientation);
-    balsa_app.page_orientation   = libbalsa_conf_get_string("PageOrientation");
-#endif
 
     g_free(balsa_app.print_header_font);
     balsa_app.print_header_font =
@@ -871,19 +854,15 @@ config_global_load(void)
                                 DEFAULT_PRINT_BODY_FONT);
     balsa_app.print_highlight_cited =
         libbalsa_conf_get_bool("PrintHighlightCited=false");
-#ifdef HAVE_GTK_PRINT
     balsa_app.print_highlight_phrases =
         libbalsa_conf_get_bool("PrintHighlightPhrases=false");
-#endif
     libbalsa_conf_pop_group();
 
-#ifdef HAVE_GTK_PRINT
     /* GtkPrint printing */
     libbalsa_conf_push_group("GtkPrint");
     libbalsa_conf_foreach_keys("GtkPrint", load_gtk_print_setting,
 			       balsa_app.print_settings);
     libbalsa_conf_pop_group();
-#endif
 
     /* Spelling options ... */
     libbalsa_conf_push_group("Spelling");
@@ -925,10 +904,8 @@ config_global_load(void)
 	d_get_gint("NewMailNotificationDialog", 0);
     balsa_app.notify_new_mail_sound =
 	d_get_gint("NewMailNotificationSound", 1);
-#if GTK_CHECK_VERSION(2, 10, 0)
     balsa_app.notify_new_mail_icon =
 	d_get_gint("NewMailNotificationIcon", 1);
-#endif                          /* GTK_CHECK_VERSION(2, 10, 0) */
     balsa_app.check_mail_upon_startup =
 	libbalsa_conf_get_bool("OnStartup=false");
     balsa_app.check_mail_auto = libbalsa_conf_get_bool("Auto=false");
@@ -1149,7 +1126,6 @@ config_global_load(void)
     return TRUE;
 }				/* config_global_load */
 
-#ifdef HAVE_GTK_PRINT
 static void
 save_gtk_print_setting(const gchar *key, const gchar *value, gpointer user_data)
 {
@@ -1195,7 +1171,6 @@ save_gtk_page_setup(GtkPageSetup *page_setup)
     libbalsa_conf_set_int("Orientation",
 			  gtk_page_setup_get_orientation(page_setup));
 }
-#endif
 
 gint
 config_save(void)
@@ -1306,7 +1281,7 @@ config_save(void)
     libbalsa_conf_set_bool("ShowMailboxList", balsa_app.show_mblist);
     libbalsa_conf_set_bool("ShowTabs", balsa_app.show_notebook_tabs);
     libbalsa_conf_set_int("ProgressWindow", balsa_app.pwindow_option);
-    libbalsa_conf_set_bool("AlternativeLayout", balsa_app.alternative_layout);
+    libbalsa_conf_set_int("LayoutType",     balsa_app.layout_type);
     libbalsa_conf_set_bool("ViewMessageOnOpen", balsa_app.view_message_on_open);
     libbalsa_conf_set_bool("PageDownMod", balsa_app.pgdownmod);
     libbalsa_conf_set_int("PageDownPercent", balsa_app.pgdown_percent);
@@ -1334,32 +1309,11 @@ config_save(void)
 
     /* Printing options ... */
     libbalsa_conf_push_group("Printing");
-#ifdef HAVE_GTK_PRINT
     save_gtk_page_setup(balsa_app.page_setup);
     libbalsa_conf_set_double("LeftMargin", balsa_app.margin_left);
     libbalsa_conf_set_double("TopMargin", balsa_app.margin_top);
     libbalsa_conf_set_double("RightMargin", balsa_app.margin_right);
     libbalsa_conf_set_double("BottomMargin", balsa_app.margin_bottom);
-#else
-    libbalsa_conf_set_string("PaperSize",balsa_app.paper_size);
-    if(balsa_app.margin_left)
-	libbalsa_conf_set_string("LeftMargin",   balsa_app.margin_left);
-    if(balsa_app.margin_top)
-	libbalsa_conf_set_string("TopMargin",    balsa_app.margin_top);
-    if(balsa_app.margin_bottom)
-	libbalsa_conf_set_string("RightMargin",  balsa_app.margin_right);
-    if(balsa_app.margin_bottom)
-	libbalsa_conf_set_string("BottomMargin", balsa_app.margin_bottom);
-    if(balsa_app.print_unit)
-	libbalsa_conf_set_string("PrintUnit",    balsa_app.print_unit);
-    if(balsa_app.print_layout)
-	libbalsa_conf_set_string("PrintLayout", balsa_app.print_layout);
-    if(balsa_app.margin_bottom)
-	libbalsa_conf_set_string("PaperOrientation", 
-				balsa_app.paper_orientation);
-    if(balsa_app.margin_bottom)
-	libbalsa_conf_set_string("PageOrientation", balsa_app.page_orientation);
-#endif
 
     libbalsa_conf_set_string("PrintHeaderFont",
                             balsa_app.print_header_font);
@@ -1368,13 +1322,10 @@ config_save(void)
                             balsa_app.print_footer_font);
     libbalsa_conf_set_bool("PrintHighlightCited",
                           balsa_app.print_highlight_cited);
-#ifdef HAVE_GTK_PRINT
     libbalsa_conf_set_bool("PrintHighlightPhrases",
                           balsa_app.print_highlight_phrases);
-#endif
     libbalsa_conf_pop_group();
 
-#ifdef HAVE_GTK_PRINT
     /* GtkPrintSettings stuff */
     libbalsa_conf_remove_group("GtkPrint");
     libbalsa_conf_push_group("GtkPrint");
@@ -1382,7 +1333,6 @@ config_save(void)
 	gtk_print_settings_foreach(balsa_app.print_settings,
 				   save_gtk_print_setting, NULL);
     libbalsa_conf_pop_group();
-#endif
 
     /* Spelling options ... */
     libbalsa_conf_remove_group("Spelling");
@@ -1418,10 +1368,8 @@ config_save(void)
                           balsa_app.notify_new_mail_dialog);
     libbalsa_conf_set_int("NewMailNotificationSound",
                           balsa_app.notify_new_mail_sound);
-#if GTK_CHECK_VERSION(2, 10, 0)
     libbalsa_conf_set_int("NewMailNotificationIcon",
                           balsa_app.notify_new_mail_icon);
-#endif                          /* GTK_CHECK_VERSION(2, 10, 0) */
     libbalsa_conf_set_bool("OnStartup", balsa_app.check_mail_upon_startup);
     libbalsa_conf_set_bool("Auto", balsa_app.check_mail_auto);
     libbalsa_conf_set_int("AutoDelay", balsa_app.check_mail_timer);
@@ -1779,7 +1727,7 @@ config_view_load(const gchar * key, const gchar * value, gpointer data)
             libbalsa_conf_get_string_with_default("MailingListAddress",
                                                   &def);
         view->mailing_list_address =
-            def ? NULL : internet_address_parse_string(address);
+            def ? NULL : internet_address_list_parse_string(address);
         g_free(address);
 
         view->identity_name = libbalsa_conf_get_string("Identity");
@@ -1968,7 +1916,6 @@ static gboolean
 config_filter_load(const gchar * key, const gchar * value, gpointer data)
 {
     char *endptr;
-    guint *save = data;
     LibBalsaFilter *fil;
     long int dummy;
 
@@ -1981,20 +1928,6 @@ config_filter_load(const gchar * key, const gchar * value, gpointer data)
     libbalsa_conf_push_group(key);
 
     fil = libbalsa_filter_new_from_config();
-    if (!fil->condition) {
-        /* Try pre-2.1 style: */
-        FilterOpType op = libbalsa_conf_get_int("Operation");
-        ConditionMatchType cmt =
-            op == FILTER_OP_OR ? CONDITION_OR : CONDITION_AND;
-        fil->condition = libbalsa_condition_new_2_0(key, cmt);
-        if (fil->condition) {
-            if (fil->action > FILTER_TRASH)
-                /* Some 2.0.x versions had a new action code which
-                 * changed the value of FILTER_TRASH. */
-                fil->action = FILTER_TRASH;
-            ++*save;
-        }
-    }
     if (!fil->condition) {
         g_idle_add((GSourceFunc) config_warning_idle,
                    _("Filter with no condition was omitted"));
@@ -2012,14 +1945,9 @@ config_filter_load(const gchar * key, const gchar * value, gpointer data)
 static void
 config_filters_load(void)
 {
-    guint save = 0;
-
     filter_errno = FILTER_NOERR;
     libbalsa_conf_foreach_group(FILTER_SECTION_PREFIX,
-                                config_filter_load, &save);
-
-    if (save)
-        config_filters_save();
+                                config_filter_load, NULL);
 }
 
 #define FILTER_SECTION_MAX "9999"
@@ -2173,6 +2101,7 @@ check_for_old_sigs(GList * id_list_tmp)
     }
 }
 
+#if HAVE_GNOME
 void
 config_defclient_save(void)
 {
@@ -2222,3 +2151,4 @@ config_defclient_save(void)
         }
     }
 }
+#endif /* HAVE_GNOME */
