@@ -19,10 +19,15 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "toolbar-factory.h"
 
 #include <string.h>
+#if HAVE_GNOME
 #include <gconf/gconf-client.h>
+#endif
 
 #include <glib/gi18n.h>
 
@@ -34,7 +39,6 @@
 #include "libbalsa-conf.h"
 
 #include "toolbar-prefs.h"
-#include "toolbar-factory.h"
 
 /* Must be consistent with BalsaToolbarType enum: */
 static const gchar *const balsa_toolbar_names[] =
@@ -263,6 +267,7 @@ tm_save_model(BalsaToolbarModel * model)
     libbalsa_conf_pop_group();
 }
 
+#if HAVE_GNOME
 /* GConfClientNotifyFunc
  */
 static void
@@ -272,6 +277,7 @@ tm_gconf_notify(GConfClient * client, guint cnxn_id, GConfEntry * entry,
     if (model->style == (GtkToolbarStyle) (-1))
         balsa_toolbar_model_changed(model);
 }
+#endif /* HAVE_GNOME */
 
 /* Create a BalsaToolbarModel structure.
  */
@@ -280,13 +286,16 @@ balsa_toolbar_model_new(BalsaToolbarType type, GSList * standard)
 {
     BalsaToolbarModel *model =
         g_object_new(BALSA_TYPE_TOOLBAR_MODEL, NULL);
+#if HAVE_GNOME
     GConfClient *conf;
     guint notify_id;
+#endif
 
     model->type = type;
     model->standard = standard;
     tm_load_model(model);
 
+#if HAVE_GNOME
     conf = gconf_client_get_default();
     /* We never destroy a model, so we do nothing with the notify-id: */
     notify_id =
@@ -294,6 +303,7 @@ balsa_toolbar_model_new(BalsaToolbarType type, GSList * standard)
                                 "/desktop/gnome/interface/toolbar_style",
                                 (GConfClientNotifyFunc) tm_gconf_notify,
                                 model, NULL, NULL);
+#endif /* HAVE_GNOME */
 
     return model;
 }
@@ -444,12 +454,10 @@ tm_set_tool_item_label(GtkToolItem * tool_item, const gchar * stock_id,
 
     gtk_tool_item_set_is_important(tool_item, TRUE);
 
-#if GTK_CHECK_VERSION(2, 12, 0)
     if (strcmp(toolbar_buttons[button].pixmap_id, BALSA_PIXMAP_SEND) == 0
         && balsa_app.always_queue_sent_mail)
         gtk_tool_item_set_tooltip_text(tool_item,
                                        _("Queue this message for sending"));
-#endif                          /* GTK_CHECK_VERSION(2, 12, 0) */
 
     return button;
 }
@@ -469,19 +477,11 @@ tm_populate(BalsaToolbarModel * model, GtkUIManager * ui_manager,
 
         g_array_append_val(merge_ids, merge_id);
 
-        if (!*stock_id) {
-#if GTK_CHECK_VERSION(2, 11, 6)
+        if (!*stock_id)
             gtk_ui_manager_add_ui(ui_manager, merge_id, "/Toolbar",
                                   NULL, NULL, GTK_UI_MANAGER_SEPARATOR,
                                   FALSE);
-#else                           /* GTK_CHECK_VERSION(2, 11, 6) */
-            gchar *name = g_strdup_printf("Separator%d", merge_id);
-            gtk_ui_manager_add_ui(ui_manager, merge_id, "/Toolbar",
-                                  name, NULL, GTK_UI_MANAGER_SEPARATOR,
-                                  FALSE);
-            g_free(name);
-#endif                          /* GTK_CHECK_VERSION(2, 11, 6) */
-        } else {
+        else {
             gchar *path, *name;
             GtkWidget *tool_item;
 
@@ -526,9 +526,10 @@ static const struct {
 static GtkToolbarStyle
 tm_default_style(void)
 {
+    GtkToolbarStyle default_style = GTK_TOOLBAR_BOTH;
+#if HAVE_GNOME
     GConfClient *conf;
     gchar *str;
-    GtkToolbarStyle default_style = GTK_TOOLBAR_BOTH;
 
     /* Get global setting */
     conf = gconf_client_get_default();
@@ -545,6 +546,7 @@ tm_default_style(void)
             }
         g_free(str);
     }
+#endif /* HAVE_GNOME */
 
     return default_style;
 }
@@ -775,9 +777,7 @@ tm_do_popup_menu(GtkWidget * toolbar, GdkEventButton * event,
         event_time = gtk_get_current_event_time();
     }
 
-#if GTK_CHECK_VERSION(2, 10, 0)
     gtk_menu_attach_to_widget(GTK_MENU(menu), toolbar, NULL);
-#endif                          /* GTK_CHECK_VERSION(2, 10, 0) */
     if (button)
         gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button,
                        event_time);
