@@ -19,7 +19,10 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
+#include "toolbar-prefs.h"
 
 #include <string.h>
 #include <glib/gi18n.h>
@@ -30,13 +33,6 @@
 #include "message-window.h"
 #include "sendmsg-window.h"
 #include "toolbar-factory.h"
-#include "toolbar-prefs.h"
-
-#if !GTK_CHECK_VERSION(2, 14, 0)
-#ifdef HAVE_GNOME
-#include <gnome.h>
-#endif
-#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
 
 #ifndef MAX
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
@@ -388,9 +384,7 @@ current_selection_changed_cb(GtkTreeSelection * selection, ToolbarPage * page)
 static void
 tp_dialog_response_cb(GtkDialog * dialog, gint response, gpointer data)
 {
-#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
     GError *err = NULL;
-#endif
 
     switch (response) {
     case GTK_RESPONSE_DELETE_EVENT:
@@ -398,20 +392,14 @@ tp_dialog_response_cb(GtkDialog * dialog, gint response, gpointer data)
         gtk_widget_destroy(GTK_WIDGET(dialog));
         break;
     case GTK_RESPONSE_HELP:
-#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
-#if GTK_CHECK_VERSION(2, 14, 0)
         gtk_show_uri(NULL, "ghelp:balsa?toolbar-prefs",
                      gtk_get_current_event_time(), &err);
-#else                           /* GTK_CHECK_VERSION(2, 14, 0) */
-        gnome_help_display("balsa", "toolbar-prefs", &err);
-#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
         if (err) {
             balsa_information(LIBBALSA_INFORMATION_WARNING,
 		    _("Error displaying toolbar help: %s\n"),
 		    err->message);
             g_error_free(err);
         }
-#endif
         break;
     default:
         break;
